@@ -28,11 +28,11 @@
 into the database when the FASL files (produced by compiling
 STORE-COVERAGE-DATA optimization policy set to 3) are loaded again into the
 image."
-  (sb-c::clear-code-coverage))
+  (sb-c:clear-code-coverage))
 
 (defun reset-coverage ()
   "Reset all coverage data back to the `Not executed` state."
-  (sb-c::reset-code-coverage))
+  (sb-c:reset-code-coverage))
 
 (defun save-coverage ()
   "Returns an opaque representation of the current code coverage state.
@@ -40,7 +40,7 @@ The only operation that may be done on the state is passing it to
 RESTORE-COVERAGE. The representation is guaranteed to be readably printable.
 A representation that has been printed and read back will work identically
 in RESTORE-COVERAGE."
-  (loop for file being the hash-keys of sb-c::*code-coverage-info*
+  (loop for file being the hash-keys of sb-c:*code-coverage-info*
         using (hash-value states)
         collect (cons file states)))
 
@@ -48,7 +48,7 @@ in RESTORE-COVERAGE."
   "Restore the code coverage data back to an earlier state produced by
 SAVE-COVERAGE."
   (loop for (file . states) in coverage-state
-        do (let ((image-states (gethash file sb-c::*code-coverage-info*))
+        do (let ((image-states (gethash file sb-c:*code-coverage-info*))
                  (table (make-hash-table :test 'equal)))
              (when image-states
                (loop for cons in image-states
@@ -93,6 +93,7 @@ result to RESTORE-COVERAGE."
                            :defaults pathname)))))
 
 (defun report (directory &key ((:form-mode *source-path-mode*) :whole)
+               (if-matches 'identity)
                (external-format :default))
   "Print a code coverage report of all instrumented files into DIRECTORY.
 If DIRECTORY does not exist, it will be created. The main report will be
@@ -103,27 +104,34 @@ If the keyword argument FORM-MODE has the value :CAR, the annotations in
 the coverage report will be placed on the CARs of any cons-forms, while if
 it has the value :WHOLE the whole form will be annotated (the default).
 The former mode shows explicitly which forms were instrumented, while the
-latter mode is generally easier to read."
+latter mode is generally easier to read.
+
+The keyword argument IF-MATCHES should be a designator for a function
+of one argument, called for the namestring of each file with code
+coverage info. If it returns true, the file's info is included in the
+report, otherwise ignored. The default value is CL:IDENTITY.
+"
   (let* ((paths)
          (directory (pathname-as-directory directory))
          (*default-pathname-defaults* (translate-logical-pathname directory)))
     (ensure-directories-exist *default-pathname-defaults*)
     (maphash (lambda (k v)
                (declare (ignore v))
-               (let* ((pk (translate-logical-pathname k))
-                      (n (format nil "~(~{~2,'0X~}~)"
-                                (coerce (sb-md5:md5sum-string
-                                         (sb-ext:native-namestring pk))
-                                        'list)))
-                      (path (make-pathname :name n :type "html" :defaults directory)))
-                 (when (probe-file k)
-                   (ensure-directories-exist pk)
-                   (with-open-file (stream path
-                                           :direction :output
-                                           :if-exists :supersede
-                                           :if-does-not-exist :create)
-                     (push (list* k n (report-file k stream external-format))
-                           paths)))))
+               (when (funcall if-matches k)
+                 (let* ((pk (translate-logical-pathname k))
+                        (n (format nil "~(~{~2,'0X~}~)"
+                                   (coerce (sb-md5:md5sum-string
+                                            (sb-ext:native-namestring pk))
+                                           'list)))
+                        (path (make-pathname :name n :type "html" :defaults directory)))
+                   (when (probe-file k)
+                     (ensure-directories-exist pk)
+                     (with-open-file (stream path
+                                             :direction :output
+                                             :if-exists :supersede
+                                             :if-does-not-exist :create)
+                       (push (list* k n (report-file k stream external-format))
+                             paths))))))
              *code-coverage-info*)
     (let ((report-file (make-pathname :name "cover-index" :type "html" :defaults directory)))
       (with-open-file (stream report-file
@@ -409,7 +417,7 @@ table.summary tr.subheading td { text-align: left; font-weight: bold; padding-le
            unless (member (caar record) '(:then :else))
            collect (list mode
                          (car record)
-                         (if (sb-c::code-coverage-record-marked record)
+                         (if (sb-c:code-coverage-record-marked record)
                              1
                              2))))
     (:branch
@@ -419,7 +427,7 @@ table.summary tr.subheading td { text-align: left; font-weight: bold; padding-le
            (when (member (car path) '(:then :else))
              (setf (gethash (cdr path) hash)
                    (logior (gethash (cdr path) hash 0)
-                           (ash (if (sb-c::code-coverage-record-marked record)
+                           (ash (if (sb-c:code-coverage-record-marked record)
                                     1
                                     2)
                                 (if (eql (car path) :then)
