@@ -40,14 +40,13 @@
 ;;; a SIMPLE-VECTOR set by GENESIS
 (defvar *!load-time-values*)
 
-(eval-when (:compile-toplevel :execute)
   ;; FIXME: Perhaps we should make SHOW-AND-CALL-AND-FMAKUNBOUND, too,
   ;; and use it for most of the cold-init functions. (Just be careful
   ;; not to use it for the COLD-INIT-OR-REINIT functions.)
-  (sb!xc:defmacro show-and-call (name)
+(defmacro show-and-call (name)
     `(progn
        (/primitive-print ,(symbol-name name))
-       (,name))))
+       (,name)))
 
 (defun !encapsulate-stuff-for-cold-init (&aux names)
   (flet ((encapsulate-1 (name handler)
@@ -130,7 +129,7 @@
   (show-and-call !random-cold-init)
 
   ;; Must be done before any non-opencoded array references are made.
-  (show-and-call !hairy-data-vector-reffer-init)
+  (show-and-call sb!vm::!hairy-data-vector-reffer-init)
 
   (show-and-call !character-database-cold-init)
   (show-and-call !character-name-database-cold-init)
@@ -174,8 +173,8 @@
       (when docstring (setf (fdocumentation name 'variable) docstring))))
   (!with-init-wrappers
    (dolist (x *!cold-defuns*)
-     (destructuring-bind (name . inline-expansion) x
-       (%defun name (fdefinition name) inline-expansion))))
+     (destructuring-bind (name inline-expansion dxable-args) x
+       (%defun name (fdefinition name) inline-expansion dxable-args))))
 
   ;; KLUDGE: Why are fixups mixed up with toplevel forms? Couldn't
   ;; fixups be done separately? Wouldn't that be clearer and better?
