@@ -66,11 +66,8 @@ provide bindings for printer control variables.")
 ;;; It actually works as long as the condition is not a subtype of WARNING
 ;;; or ERROR. (Any other direct descendant of CONDITION should be fine)
 (!defvar *stack-top-hint* nil)
-
-(defvar *real-stack-top* nil)
-(defvar *stack-top* nil)
-
 (defvar *current-frame* nil)
+(declaim (always-bound *stack-top-hint* *current-frame*))
 
 ;;; Beginner-oriented help messages are important because you end up
 ;;; in the debugger whenever something bad happens, or if you try to
@@ -83,7 +80,7 @@ provide bindings for printer control variables.")
   (sb!thread::get-foreground)
   (format stream
           "~%~W~:[~;[~W~]] "
-          (sb!di:frame-number *current-frame*)
+          (frame-number *current-frame*)
           (> *debug-command-level* 1)
           *debug-command-level*))
 
@@ -162,11 +159,11 @@ Other commands:
 ;;; location. Used by source printing to some information instead of
 ;;; none for the user.
 (defun maybe-block-start-location (loc)
-  (if (sb!di:code-location-unknown-p loc)
-      (let* ((block (sb!di:code-location-debug-block loc))
-             (start (sb!di:do-debug-block-locations (loc block)
+  (if (code-location-unknown-p loc)
+      (let* ((block (code-location-debug-block loc))
+             (start (do-debug-block-locations (loc block)
                       (return loc))))
-        (cond ((and (not (sb!di:debug-block-elsewhere-p block))
+        (cond ((and (not (debug-block-elsewhere-p block))
                     start)
                (format *debug-io* "~%unknown location: using block start~%")
                start)
@@ -210,7 +207,7 @@ backtraces. Possible values are :MINIMAL, :NORMAL, and :FULL.
    In the this case arguments may include values internal to SBCL's method
    dispatch machinery.")
 
-(define-deprecated-variable :early "1.1.4.9" *show-entry-point-details*
+(define-deprecated-variable :late "1.1.4.9" *show-entry-point-details*
   :value nil)
 
 (define-deprecated-function :early "1.2.15" backtrace (print-backtrace)
@@ -222,12 +219,12 @@ backtraces. Possible values are :MINIMAL, :NORMAL, and :FULL.
   (list-backtrace :count count))
 
 (defun backtrace-start-frame (frame-designator)
-  (let ((here (sb!di:top-frame)))
+  (let ((here (top-frame)))
     (labels ((current-frame ()
                (let ((frame here))
                  ;; Our caller's caller.
                  (loop repeat 2
-                       do (setf frame (or (sb!di:frame-down frame) frame)))
+                       do (setf frame (or (frame-down frame) frame)))
                  frame))
              (interrupted-frame ()
                (or (find-interrupted-frame)
@@ -240,7 +237,7 @@ backtraces. Possible values are :MINIMAL, :NORMAL, and :FULL.
             (if (and *in-the-debugger* *current-frame*)
                 *current-frame*
                 (interrupted-frame)))
-           ((sb!di:frame-p frame-designator)
+           ((frame-p frame-designator)
             frame-designator)
            (t
             (error "Invalid designator for initial backtrace frame: ~S"
@@ -275,10 +272,11 @@ is :DEBUGGER-FRAME.
     specifies the currently debugged frame when inside the debugger, and
     behaves as :INTERRUPTED-FRAME outside the debugger.
 "
+  (declare (dynamic-extent function))
   (loop with result = nil
         for index upfrom 0
         for frame = (backtrace-start-frame from)
-        then (sb!di:frame-down frame)
+        then (frame-down frame)
         until (null frame)
         when (<= start index) do
         (if (minusp (decf count))
@@ -442,6 +440,16 @@ information."
                         :replace-dynamic-extent-objects t)
     (values (cons name args) info)))
 
+;;; This is used in constructing arg lists for debugger printing,
+;;; and when needing to print unbound slots in PCL.
+(defstruct (unprintable-object
+            (:constructor make-unprintable-object (string))
+            (:print-object (lambda (x s)
+                             (print-unreadable-object (x s)
+                               (write-string (unprintable-object-string x) s))))
+            (:copier nil))
+  (string nil :read-only t))
+
 (defun replace-dynamic-extent-object (obj)
   (if (stack-allocated-p obj)
       (make-unprintable-object
@@ -454,10 +462,9 @@ information."
 (defun stack-allocated-p (obj)
   "Returns T if OBJ is allocated on the stack of the current
 thread, NIL otherwise."
-  (with-pinned-objects (obj)
-    (let ((sap (int-sap (get-lisp-obj-address obj))))
-      (when (sb!vm:control-stack-pointer-valid-p sap nil)
-        t))))
+  (let ((sap (int-sap (get-lisp-obj-address obj))))
+    (when (sb!vm:control-stack-pointer-valid-p sap nil)
+      t)))
 
 ;;;; frame printing
 
@@ -474,7 +481,7 @@ thread, NIL otherwise."
                                               more
                                               deleted)
   `(etypecase ,element
-     (sb!di:debug-var
+     (debug-var
       ,@required)
      (cons
       (ecase (car ,element)
@@ -490,7 +497,7 @@ thread, NIL otherwise."
   (let ((var (gensym)))
     `(let ((,var ,variable))
        (cond ((eq ,var :deleted) ,deleted)
-             ((eq (sb!di:debug-var-validity ,var ,location) :valid)
+             ((eq (debug-var-validity ,var ,location) :valid)
               ,valid)
              (t ,other)))))
 
@@ -498,14 +505,14 @@ thread, NIL otherwise."
 
 ;;; Extract the function argument values for a debug frame.
 (defun map-frame-args (thunk frame)
-  (let ((debug-fun (sb!di:frame-debug-fun frame)))
-    (dolist (element (sb!di:debug-fun-lambda-list debug-fun))
+  (let ((debug-fun (frame-debug-fun frame)))
+    (dolist (element (debug-fun-lambda-list debug-fun))
       (funcall thunk element))))
 
 ;;; When the frame is interrupted before any of the function code is called
 ;;; we can recover all the arguments, include the extra ones.
-;;; This includes the ARG-COUNT-ERROR on #+precise-arg-count-error and
-;;; UNDEFINED-FUNCTION coming from undefined-tramp
+;;; This includes the ARG-COUNT-ERROR and UNDEFINED-FUNCTION coming from
+;;; undefined-tramp.
 (defun early-frame-nth-arg (n frame)
   (let* ((escaped (sb!di::compiled-frame-escaped frame))
          (pointer (sb!di::frame-pointer frame))
@@ -535,7 +542,7 @@ thread, NIL otherwise."
     (return-from frame-args-as-list
       (early-frame-args frame)))
   (handler-case
-      (let ((location (sb!di:frame-code-location frame))
+      (let ((location (frame-code-location frame))
             (reversed-result nil))
         (block enumerating
           (map-frame-args
@@ -550,7 +557,7 @@ thread, NIL otherwise."
               :deleted ((push (frame-call-arg element location frame) reversed-result))
               :rest ((lambda-var-dispatch (second element) location
                       nil
-                      (let ((rest (sb!di:debug-var-value (second element) frame)))
+                      (let ((rest (debug-var-value (second element) frame)))
                         (if (listp rest)
                             (setf reversed-result (append (reverse rest) reversed-result))
                             (push (make-unprintable-object "unavailable &REST argument")
@@ -561,8 +568,8 @@ thread, NIL otherwise."
                             reversed-result)))
               :more ((lambda-var-dispatch (second element) location
                       nil
-                      (let ((context (sb!di:debug-var-value (second element) frame))
-                            (count (sb!di:debug-var-value (third element) frame)))
+                      (let ((context (debug-var-value (second element) frame))
+                            (count (debug-var-value (third element) frame)))
                         (setf reversed-result
                               (append (reverse
                                        (multiple-value-list
@@ -573,29 +580,11 @@ thread, NIL otherwise."
                             reversed-result)))))
            frame))
         (nreverse reversed-result))
-    (sb!di:lambda-list-unavailable ()
+    (lambda-list-unavailable ()
       (make-unprintable-object "unavailable lambda list"))))
 
 (defun clean-xep (frame name args info)
   (values name
-          #!-precise-arg-count-error
-          (if (consp args)
-              (let* ((count (first args))
-                     (real-args (rest args)))
-                (if (and (integerp count)
-                         (sb!di::all-args-available-p frame))
-                    ;; So, this is a cheap trick -- but makes backtraces for
-                    ;; too-many-arguments-errors much, much easier to to
-                    ;; understand.
-                    (loop repeat count
-                          for arg = (if real-args
-                                        (pop real-args)
-                                        (make-unprintable-object "unknown"))
-                          collect arg)
-                    real-args))
-              args)
-          ;; Clip arg-count.
-          #!+precise-arg-count-error
           (if (and (consp args)
                    ;; EARLY-FRAME-ARGS doesn't include arg-count
                    (not (sb!di::all-args-available-p frame)))
@@ -630,7 +619,7 @@ thread, NIL otherwise."
                 (real-args (the list (cddr args)))) ; strip .PV. and .N-M-CALL.
             (if (and (eq style :minimal)
                      (fboundp gf-name)
-                     (notany #'sb!impl::unprintable-object-p real-args)
+                     (notany #'unprintable-object-p real-args)
                      (singleton-p (compute-applicable-methods
                                    (fdefinition gf-name) real-args)))
                 (values gf-name real-args)
@@ -639,13 +628,12 @@ thread, NIL otherwise."
 
 (defun clean-frame-call (frame name method-frame-style info)
   (let ((args (frame-args-as-list frame)))
-    (cond ((memq :external info)
+    (cond ((typep name '(cons (eql sb!pcl::fast-method)))
+           (clean-fast-method name args method-frame-style info))
+          ((memq :external info)
            (clean-xep frame name args info))
           ((memq :more info)
            (clean-&more-processor name args info))
-          ((and (consp name)
-                (eq (car name) 'sb!pcl::fast-method))
-           (clean-fast-method name args method-frame-style info))
           (t
            (values name args info)))))
 
@@ -665,12 +653,12 @@ information.
 
 If REPLACE-DYNAMIC-EXTENT-OBJECTS is true, objects allocated on the stack of
 the current thread are replaced with dummy objects which can safely escape."
-  (let* ((debug-fun (sb!di:frame-debug-fun frame))
-         (kind (sb!di:debug-fun-kind debug-fun)))
+  (let* ((debug-fun (frame-debug-fun frame))
+         (kind (debug-fun-kind debug-fun)))
     (multiple-value-bind (name args info)
         (clean-frame-call frame
-                          (or (sb!di:debug-fun-closure-name debug-fun frame)
-                              (sb!di:debug-fun-name debug-fun))
+                          (or (debug-fun-closure-name debug-fun frame)
+                              (debug-fun-name debug-fun))
                           method-frame-style
                           (when kind (list kind)))
       (let ((args (if (and (consp args) replace-dynamic-extent-objects)
@@ -690,7 +678,7 @@ the current thread are replaced with dummy objects which can safely escape."
 (defun frame-call-arg (var location frame)
   (lambda-var-dispatch var location
     (make-unprintable-object "unused argument")
-    (sb!di:debug-var-value var frame)
+    (debug-var-value var frame)
     (make-unprintable-object "unavailable argument")))
 
 ;;; Prints a representation of the function call causing FRAME to
@@ -706,7 +694,7 @@ the current thread are replaced with dummy objects which can safely escape."
   (when number
     (format stream "~&~S: " (if (integerp number)
                                 number
-                                (sb!di:frame-number frame))))
+                                (frame-number frame))))
   (multiple-value-bind (name args info)
       (frame-call frame :method-frame-style method-frame-style)
     (pprint-logical-block (stream nil :prefix "(" :suffix ")")
@@ -731,19 +719,23 @@ the current thread are replaced with dummy objects which can safely escape."
         (let ((args (if emergency-best-effort
                         (ensure-printable-object args)
                         args)))
-          (if (listp args)
-              (format stream "~{ ~_~S~}" args)
-              (format stream " ~S" args)))))
+          (cond ((not (listp args))
+                 (format stream " ~S" args))
+                (t
+                 (dolist (arg args)
+                   (write-char #\space stream)
+                   (pprint-newline :linear stream)
+                   (write arg :stream stream :escape t)))))))
     (when info
       (format stream " [~{~(~A~)~^,~}]" info)))
   (when print-frame-source
-    (let* ((loc (sb!di:frame-code-location frame))
+    (let* ((loc (frame-code-location frame))
            (path (and (sb!di::compiled-debug-fun-p
-                       (sb!di:code-location-debug-fun loc))
-                      (handler-case (sb!di:code-location-debug-source loc)
-                        (sb!di:no-debug-blocks ())
+                       (code-location-debug-fun loc))
+                      (handler-case (code-location-debug-source loc)
+                        (no-debug-blocks ())
                         (:no-error (source)
-                          (sb!di:debug-source-namestring source))))))
+                          (debug-source-namestring source))))))
       (when (or (eq print-frame-source :always)
                 ;; Avoid showing sources for internals,
                 ;; it will either fail anyway due to all the SB! and
@@ -756,7 +748,7 @@ the current thread are replaced with dummy objects which can safely escape."
                             (error (c)
                               (format stream "~&   error finding frame source: ~A" c)))))
               (format stream "~%   source: ~S" source))
-          (sb!di:debug-condition ()
+          (debug-condition ()
             ;; This is mostly noise.
             (when (eq :always print-frame-source)
               (format stream "~&   no source available for frame")))
@@ -772,17 +764,6 @@ the current thread are replaced with dummy objects which can safely escape."
    of this variable to the function because it binds *DEBUGGER-HOOK* to NIL
    around the invocation.")
 
-(defvar *invoke-debugger-hook* nil
-  "This is either NIL or a designator for a function of two arguments,
-   to be run when the debugger is about to be entered.  The function is
-   run with *INVOKE-DEBUGGER-HOOK* bound to NIL to minimize recursive
-   errors, and receives as arguments the condition that triggered
-   debugger entry and the previous value of *INVOKE-DEBUGGER-HOOK*
-
-   This mechanism is an SBCL extension similar to the standard *DEBUGGER-HOOK*.
-   In contrast to *DEBUGGER-HOOK*, it is observed by INVOKE-DEBUGGER even when
-   called by BREAK.")
-
 ;;; These are bound on each invocation of INVOKE-DEBUGGER.
 (defvar *debug-restarts*)
 (defvar *debug-condition*)
@@ -884,7 +865,8 @@ the current thread are replaced with dummy objects which can safely escape."
 
 (defun invoke-debugger (condition)
   "Enter the debugger."
-  (let ((*stack-top-hint* (resolve-stack-top-hint)))
+  (let ((*stack-top-hint* (resolve-stack-top-hint))
+        (sb!impl::*deadline* nil))
     ;; call *INVOKE-DEBUGGER-HOOK* first, so that *DEBUGGER-HOOK* is not
     ;; called when the debugger is disabled
     (run-hook '*invoke-debugger-hook* condition)
@@ -962,34 +944,46 @@ the current thread are replaced with dummy objects which can safely escape."
       ;; older debugger code which was written to do i/o on whatever
       ;; stream was in fashion at the time, and not all of it has
       ;; been converted to behave this way. -- WHN 2000-11-16)
-
-      (unwind-protect
-           (let (;; We used to bind *STANDARD-OUTPUT* to *DEBUG-IO*
-                 ;; here as well, but that is probably bogus since it
-                 ;; removes the users ability to do output to a redirected
-                 ;; *S-O*. Now we just rebind it so that users can temporarily
-                 ;; frob it. FIXME: This and other "what gets bound when"
-                 ;; behaviour should be documented in the manual.
-                 (*standard-output* *standard-output*)
-                 ;; This seems reasonable: e.g. if the user has redirected
-                 ;; *ERROR-OUTPUT* to some log file, it's probably wrong
-                 ;; to send errors which occur in interactive debugging to
-                 ;; that file, and right to send them to *DEBUG-IO*.
-                 (*error-output* *debug-io*))
-             (unless (typep condition 'step-condition)
-               (when *debug-beginner-help-p*
-                 (format *debug-io*
-                         "~%~@<Type HELP for debugger help, or ~
+      (flet ((debug ()
+               (unwind-protect
+                    (let ( ;; We used to bind *STANDARD-OUTPUT* to *DEBUG-IO*
+                          ;; here as well, but that is probably bogus since it
+                          ;; removes the users ability to do output to a redirected
+                          ;; *S-O*. Now we just rebind it so that users can temporarily
+                          ;; frob it. FIXME: This and other "what gets bound when"
+                          ;; behaviour should be documented in the manual.
+                          (*standard-output* *standard-output*)
+                          ;; This seems reasonable: e.g. if the user has redirected
+                          ;; *ERROR-OUTPUT* to some log file, it's probably wrong
+                          ;; to send errors which occur in interactive debugging to
+                          ;; that file, and right to send them to *DEBUG-IO*.
+                          (*error-output* *debug-io*))
+                      (unless (typep condition 'step-condition)
+                        (when *debug-beginner-help-p*
+                          (format *debug-io*
+                                  "~%~@<Type HELP for debugger help, or ~
                                (SB-EXT:EXIT) to exit from SBCL.~:@>~2%"))
-               (show-restarts *debug-restarts* *debug-io*))
-             (internal-debug))
-        (when background-p
-          (sb!thread::release-foreground))))))
+                        (show-restarts *debug-restarts* *debug-io*))
+                      (internal-debug))
+                 (when background-p
+                   (sb!thread:release-foreground)))))
+        (if (find 'abort *debug-restarts* :key #'restart-name)
+            (debug)
+            (restart-case (let* ((restarts (compute-restarts condition))
+                                 ;; Put the ABORT restart last,
+                                 ;; as if it were provided by a toplevel function.
+                                 (*debug-restarts* (nconc (cdr restarts)
+                                                          (list (car restarts)))))
+                            (debug))
+              (abort ()
+                :report (lambda (stream)
+                          (format stream "~@<Exit from the current thread.~@:>"))
+                (sb!thread:abort-thread :allow-exit t))))))))
 
 ;;; this function is for use in *INVOKE-DEBUGGER-HOOK* when ordinary
 ;;; ANSI behavior has been suppressed by the "--disable-debugger"
 ;;; command-line option
-(defun debugger-disabled-hook (condition previous-hook)
+(defun debugger-disabled-hook (condition previous-hook &key (quit t))
   (declare (ignore previous-hook))
   ;; There is no one there to interact with, so report the
   ;; condition and terminate the program.
@@ -1057,7 +1051,8 @@ the current thread are replaced with dummy objects which can safely escape."
             (format *error-output*
                     "~%unhandled condition in --disable-debugger mode, quitting~%")
             (finish-output *error-output*)
-            (failure-quit))
+            (when quit
+              (failure-quit)))
         (condition ()
           ;; We IGNORE-ERRORS here because even %PRIMITIVE PRINT can
           ;; fail when our output streams are blown away, as e.g. when
@@ -1180,11 +1175,9 @@ and LDB (the low-level debugger).  See also ENABLE-DEBUGGER."
 
 (defun debug-loop-fun ()
   (let* ((*debug-command-level* (1+ *debug-command-level*))
-         (*real-stack-top* (sb!di:top-frame))
-         (*stack-top* (or *stack-top-hint* *real-stack-top*))
-         (*stack-top-hint* nil)
-         (*current-frame* *stack-top*))
-    (handler-bind ((sb!di:debug-condition
+         (*current-frame* (or *stack-top-hint* (top-frame)))
+         (*stack-top-hint* nil))
+    (handler-bind ((debug-condition
                     (lambda (condition)
                       (princ condition *debug-io*)
                       (/show0 "handling d-c by THROWing DEBUG-LOOP-CATCHER")
@@ -1241,7 +1234,7 @@ forms that explicitly control this kind of evaluation.")
   (cond ((not (and (fboundp 'compile) *auto-eval-in-frame*))
          (eval expr))
         ((frame-has-debug-vars-p *current-frame*)
-         (sb!di:eval-in-frame *current-frame* expr))
+         (eval-in-frame *current-frame* expr))
         (t
          (format *debug-io* "; No debug variables for current frame: ~
                                using EVAL instead of EVAL-IN-FRAME.~%")
@@ -1264,22 +1257,22 @@ forms that explicitly control this kind of evaluation.")
 
 (defun var-valid-in-frame-p (var location &optional (frame *current-frame*))
   ;; arg count errors are checked before anything is set up but they
-  ;; are reporeted in *elsewhere*, which is after start-pc saved in the
+  ;; are reported in the elsewhere segment, which is after start-pc saved in the
   ;; debug function, defeating the checks.
   (and (not (sb!di::all-args-available-p frame))
-       (eq (sb!di:debug-var-validity var location) :valid)))
+       (eq (debug-var-validity var location) :valid)))
 
 (eval-when (:execute :compile-toplevel)
 
 (sb!xc:defmacro define-var-operation (ref-or-set &optional value-var)
   `(let* ((temp (etypecase name
-                  (symbol (sb!di:debug-fun-symbol-vars
-                           (sb!di:frame-debug-fun *current-frame*)
+                  (symbol (debug-fun-symbol-vars
+                           (frame-debug-fun *current-frame*)
                            name))
-                  (simple-string (sb!di:ambiguous-debug-vars
-                                  (sb!di:frame-debug-fun *current-frame*)
+                  (simple-string (ambiguous-debug-vars
+                                  (frame-debug-fun *current-frame*)
                                   name))))
-          (location (sb!di:frame-code-location *current-frame*))
+          (location (frame-code-location *current-frame*))
           ;; Let's only deal with valid variables.
           (vars (remove-if-not (lambda (v)
                                  (var-valid-in-frame-p v location))
@@ -1290,9 +1283,9 @@ forms that explicitly control this kind of evaluation.")
            ((= (length vars) 1)
             ,(ecase ref-or-set
                (:ref
-                '(sb!di:debug-var-value (car vars) *current-frame*))
+                '(debug-var-value (car vars) *current-frame*))
                (:set
-                `(setf (sb!di:debug-var-value (car vars) *current-frame*)
+                `(setf (debug-var-value (car vars) *current-frame*)
                        ,value-var))))
            (t
             ;; Since we have more than one, first see whether we have
@@ -1303,7 +1296,7 @@ forms that explicitly control this kind of evaluation.")
                    ;; FIXME: REMOVE-IF-NOT is deprecated, use STRING/=
                    ;; instead.
                    (exact (remove-if-not (lambda (v)
-                                           (string= (sb!di:debug-var-symbol-name v)
+                                           (string= (debug-var-symbol-name v)
                                                     name))
                                          vars))
                    (vars (or exact vars)))
@@ -1314,9 +1307,9 @@ forms that explicitly control this kind of evaluation.")
                ((= (length vars) 1)
                 ,(ecase ref-or-set
                    (:ref
-                    '(sb!di:debug-var-value (car vars) *current-frame*))
+                    '(debug-var-value (car vars) *current-frame*))
                    (:set
-                    `(setf (sb!di:debug-var-value (car vars) *current-frame*)
+                    `(setf (debug-var-value (car vars) *current-frame*)
                            ,value-var))))
                ;; If there weren't any exact matches, flame about
                ;; ambiguity unless all the variables have the same
@@ -1324,33 +1317,33 @@ forms that explicitly control this kind of evaluation.")
                ((and (not exact)
                      (find-if-not
                       (lambda (v)
-                        (string= (sb!di:debug-var-symbol-name v)
-                                 (sb!di:debug-var-symbol-name (car vars))))
+                        (string= (debug-var-symbol-name v)
+                                 (debug-var-symbol-name (car vars))))
                       (cdr vars)))
                 (error "specification ambiguous:~%~{   ~A~%~}"
-                       (mapcar #'sb!di:debug-var-symbol-name
+                       (mapcar #'debug-var-symbol-name
                                (delete-duplicates
                                 vars :test #'string=
-                                :key #'sb!di:debug-var-symbol-name))))
+                                :key #'debug-var-symbol-name))))
                ;; All names are the same, so see whether the user
                ;; ID'ed one of them.
                (id-supplied
-                (let ((v (find id vars :key #'sb!di:debug-var-id)))
+                (let ((v (find id vars :key #'debug-var-id)))
                   (unless v
                     (error
                      "invalid variable ID, ~W: should have been one of ~S"
                      id
-                     (mapcar #'sb!di:debug-var-id vars)))
+                     (mapcar #'debug-var-id vars)))
                   ,(ecase ref-or-set
                      (:ref
-                      '(sb!di:debug-var-value v *current-frame*))
+                      '(debug-var-value v *current-frame*))
                      (:set
-                      `(setf (sb!di:debug-var-value v *current-frame*)
+                      `(setf (debug-var-value v *current-frame*)
                              ,value-var)))))
                (t
                 (error "Specify variable ID to disambiguate ~S. Use one of ~S."
                        name
-                       (mapcar #'sb!di:debug-var-id vars)))))))))
+                       (mapcar #'debug-var-id vars)))))))))
 
 ) ; EVAL-WHEN
 
@@ -1399,11 +1392,11 @@ forms that explicitly control this kind of evaluation.")
                          (return (values (third ele) t)))))
         :deleted ((if (zerop n) (return (values ele t))))
         :rest ((let ((var (second ele)))
-                 (lambda-var-dispatch var (sb!di:frame-code-location
+                 (lambda-var-dispatch var (frame-code-location
                                            *current-frame*)
                    (error "unused &REST argument before n'th argument")
                    (dolist (value
-                            (sb!di:debug-var-value var *current-frame*)
+                            (debug-var-value var *current-frame*)
                             (error
                              "The argument specification ~S is out of range."
                              n))
@@ -1421,14 +1414,14 @@ forms that explicitly control this kind of evaluation.")
     (return-from arg
       (early-frame-nth-arg n *current-frame*)))
   (multiple-value-bind (var lambda-var-p)
-      (nth-arg n (handler-case (sb!di:debug-fun-lambda-list
-                                (sb!di:frame-debug-fun *current-frame*))
-                   (sb!di:lambda-list-unavailable ()
+      (nth-arg n (handler-case (debug-fun-lambda-list
+                                (frame-debug-fun *current-frame*))
+                   (lambda-list-unavailable ()
                      (error "No argument values are available."))))
     (if lambda-var-p
-        (lambda-var-dispatch var (sb!di:frame-code-location *current-frame*)
+        (lambda-var-dispatch var (frame-code-location *current-frame*)
           (error "Unused arguments have no values.")
-          (sb!di:debug-var-value var *current-frame*)
+          (debug-var-value var *current-frame*)
           (error "invalid argument value"))
         var)))
 
@@ -1524,7 +1517,7 @@ forms that explicitly control this kind of evaluation.")
 ;;;; frame-changing commands
 
 (!def-debug-command "UP" ()
-  (let ((next (sb!di:frame-up *current-frame*)))
+  (let ((next (frame-up *current-frame*)))
     (cond (next
            (setf *current-frame* next)
            (print-frame-call next *debug-io*))
@@ -1532,7 +1525,7 @@ forms that explicitly control this kind of evaluation.")
            (format *debug-io* "~&Top of stack.")))))
 
 (!def-debug-command "DOWN" ()
-  (let ((next (sb!di:frame-down *current-frame*)))
+  (let ((next (frame-down *current-frame*)))
     (cond (next
            (setf *current-frame* next)
            (print-frame-call next *debug-io*))
@@ -1543,7 +1536,7 @@ forms that explicitly control this kind of evaluation.")
 
 (!def-debug-command "BOTTOM" ()
   (do ((prev *current-frame* lead)
-       (lead (sb!di:frame-down *current-frame*) (sb!di:frame-down lead)))
+       (lead (frame-down *current-frame*) (frame-down lead)))
       ((null lead)
        (setf *current-frame* prev)
        (print-frame-call prev *debug-io*))))
@@ -1554,11 +1547,11 @@ forms that explicitly control this kind of evaluation.")
                              (n (read-prompting-maybe "frame number: ")))
   (setf *current-frame*
         (multiple-value-bind (next-frame-fun limit-string)
-            (if (< n (sb!di:frame-number *current-frame*))
-                (values #'sb!di:frame-up "top")
-              (values #'sb!di:frame-down "bottom"))
+            (if (< n (frame-number *current-frame*))
+                (values #'frame-up "top")
+              (values #'frame-down "bottom"))
           (do ((frame *current-frame*))
-              ((= n (sb!di:frame-number frame))
+              ((= n (frame-number frame))
                frame)
             (let ((next-frame (funcall next-frame-fun frame)))
               (cond (next-frame
@@ -1633,23 +1626,23 @@ forms that explicitly control this kind of evaluation.")
 (!def-debug-command-alias "P" "PRINT")
 
 (!def-debug-command "LIST-LOCALS" ()
-  (let ((d-fun (sb!di:frame-debug-fun *current-frame*)))
+  (let ((d-fun (frame-debug-fun *current-frame*)))
     #!+sb-fasteval
-    (when (typep (sb!di:debug-fun-name d-fun nil)
+    (when (typep (debug-fun-name d-fun nil)
                  '(cons (eql sb!interpreter::.eval.)))
       (let ((env (arg 1)))
         (when (typep env 'sb!interpreter:basic-env)
           (return-from list-locals-debug-command
             (sb!interpreter:list-locals env)))))
-    (if (sb!di:debug-var-info-available d-fun)
+    (if (debug-var-info-available d-fun)
         (let ((*standard-output* *debug-io*)
-              (location (sb!di:frame-code-location *current-frame*))
+              (location (frame-code-location *current-frame*))
               (prefix (read-if-available nil))
               (any-p nil)
               (any-valid-p nil)
               (more-context nil)
               (more-count nil))
-          (dolist (v (sb!di:ambiguous-debug-vars
+          (dolist (v (ambiguous-debug-vars
                       d-fun
                       (if prefix (string prefix) "")))
             (setf any-p t)
@@ -1657,15 +1650,15 @@ forms that explicitly control this kind of evaluation.")
               (setf any-valid-p t)
               (case (sb!di::debug-var-info v)
                 (:more-context
-                 (setf more-context (sb!di:debug-var-value v *current-frame*)))
+                 (setf more-context (debug-var-value v *current-frame*)))
                 (:more-count
-                 (setf more-count (sb!di:debug-var-value v *current-frame*)))
+                 (setf more-count (debug-var-value v *current-frame*)))
                 (t
                  (format *debug-io* "~S~:[#~W~;~*~]  =  ~S~%"
-                         (sb!di:debug-var-symbol v)
-                         (zerop (sb!di:debug-var-id v))
-                         (sb!di:debug-var-id v)
-                         (sb!di:debug-var-value v *current-frame*))))))
+                         (debug-var-symbol v)
+                         (zerop (debug-var-id v))
+                         (debug-var-id v)
+                         (debug-var-value v *current-frame*))))))
           (when (and more-context more-count)
             (format *debug-io* "~S  =  ~S~%"
                     'more
@@ -1687,7 +1680,7 @@ forms that explicitly control this kind of evaluation.")
 (!def-debug-command-alias "L" "LIST-LOCALS")
 
 (!def-debug-command "SOURCE" ()
-  (print (code-location-source-form (sb!di:frame-code-location *current-frame*)
+  (print (code-location-source-form (frame-code-location *current-frame*)
                                     (read-if-available 0))
          *debug-io*))
 
@@ -1695,12 +1688,12 @@ forms that explicitly control this kind of evaluation.")
 
 (defun code-location-source-form (location context &optional (errorp t))
   (let* ((start-location (maybe-block-start-location location))
-         (form-num (sb!di:code-location-form-number start-location)))
+         (form-num (code-location-form-number start-location)))
     (multiple-value-bind (translations form)
-        (sb!di:get-toplevel-form start-location)
+        (get-toplevel-form start-location)
       (declare (notinline warn))
       (cond ((< form-num (length translations))
-             (sb!di:source-path-context form
+             (source-path-context form
                                         (svref translations form-num)
                                         context))
             (t
@@ -1747,9 +1740,9 @@ forms that explicitly control this kind of evaluation.")
 ;;; miscellaneous commands
 
 (!def-debug-command "DESCRIBE" ()
-  (let* ((curloc (sb!di:frame-code-location *current-frame*))
-         (debug-fun (sb!di:code-location-debug-fun curloc))
-         (function (sb!di:debug-fun-fun debug-fun)))
+  (let* ((curloc (frame-code-location *current-frame*))
+         (debug-fun (code-location-debug-fun curloc))
+         (function (debug-fun-fun debug-fun)))
     (if function
         (describe function)
         (format *debug-io* "can't figure out the function for this frame"))))
@@ -1791,15 +1784,15 @@ forms that explicitly control this kind of evaluation.")
                            (funcall thunk)))))
   #!-unwind-to-frame-and-call-vop
   (let ((tag (gensym)))
-    (sb!di:replace-frame-catch-tag frame
+    (replace-frame-catch-tag frame
                                    'sb!c:debug-catch-tag
                                    tag)
     (throw tag thunk)))
 
 #!+unwind-to-frame-and-call-vop
 (defun find-binding-stack-pointer (frame)
-  (let ((debug-fun (sb!di:frame-debug-fun frame)))
-    (if (eq (sb!di:debug-fun-kind debug-fun) :external)
+  (let ((debug-fun (frame-debug-fun frame)))
+    (if (eq (debug-fun-kind debug-fun) :external)
         ;; XEPs do not bind anything, nothing to restore.
         ;; But they may call other code through SATISFIES
         ;; declaration, check that the interrupt is actually in the XEP.
@@ -1854,9 +1847,9 @@ forms that explicitly control this kind of evaluation.")
                               (return (read-prompting-maybe
                                        "return: ")))
    (if (frame-has-debug-tag-p *current-frame*)
-       (let* ((code-location (sb!di:frame-code-location *current-frame*))
+       (let* ((code-location (frame-code-location *current-frame*))
               (values (multiple-value-list
-                       (funcall (sb!di:preprocess-for-eval return code-location)
+                       (funcall (preprocess-for-eval return code-location)
                                 *current-frame*))))
          (unwind-to-frame-and-call *current-frame* (lambda ()
                                                      (values-list values))))
@@ -1871,7 +1864,7 @@ forms that explicitly control this kind of evaluation.")
         (multiple-value-bind (fun arglist ok)
             (if (and (legal-fun-name-p fname) (fboundp fname))
                 (values (fdefinition fname) args t)
-                (values (sb!di:debug-fun-fun (sb!di:frame-debug-fun *current-frame*))
+                (values (debug-fun-fun (frame-debug-fun *current-frame*))
                         (frame-args-as-list *current-frame*)
                         nil))
           (when (and fun
@@ -1903,19 +1896,19 @@ forms that explicitly control this kind of evaluation.")
   (find 'sb!c:debug-catch-tag (sb!di::frame-catches frame) :key #'car))
 
 (defun frame-has-debug-vars-p (frame)
-  (sb!di:debug-var-info-available
-   (sb!di:code-location-debug-fun
-    (sb!di:frame-code-location frame))))
+  (debug-var-info-available
+   (code-location-debug-fun
+    (frame-code-location frame))))
 
 ;;;; debug loop command utilities
 
 (defun read-prompting-maybe (prompt)
-  (unless (sb!int:listen-skip-whitespace *debug-io*)
+  (unless (listen-skip-whitespace *debug-io*)
     (princ prompt *debug-io*)
     (force-output *debug-io*))
   (read *debug-io*))
 
 (defun read-if-available (default)
-  (if (sb!int:listen-skip-whitespace *debug-io*)
+  (if (listen-skip-whitespace *debug-io*)
       (read *debug-io*)
       default))
