@@ -127,25 +127,6 @@
 
 ;;;; miscellaneous fops
 
-;;; Setting this variable causes execution of a FOP-NOP4 to produce
-;;; output to *DEBUG-IO*. This can be handy when trying to follow the
-;;; progress of FASL loading.
-#!+sb-show
-(defvar *show-fop-nop4-p* nil)
-
-;;; CMU CL had a single no-op fop, FOP-NOP, with fop code 0. Since 0
-;;; occurs disproportionately often in fasl files for other reasons,
-;;; FOP-NOP is less than ideal for writing human-readable patterns
-;;; into fasl files for debugging purposes. There's no shortage of
-;;; unused fop codes, so we add this second NOP, which reads 4
-;;; arbitrary bytes and discards them.
-(!define-fop 137 (fop-nop4 () nil)
-  (let ((arg (read-arg 4 (fasl-input-stream))))
-    (declare (ignorable arg))
-    #!+sb-show
-    (when *show-fop-nop4-p*
-      (format *debug-io* "~&/FOP-NOP4 ARG=~W=#X~X~%" arg arg))))
-
 (!define-fop 0 (fop-nop () nil))
 (!define-fop 1 (fop-pop (x) nil) (push-fop-table x (fasl-input)))
 (!define-fop 2 (fop-empty-list) nil)
@@ -371,7 +352,7 @@
     (define-float-fop 52 fop-long-float long-float)))
 
 #!+sb-simd-pack
-(!define-fop 88 (fop-simd-pack)
+(!define-fop 88 :not-host (fop-simd-pack)
   (with-fast-read-byte ((unsigned-byte 8) (fasl-input-stream))
     (%make-simd-pack (fast-read-s-integer 8)
                      (fast-read-u-integer 8)
@@ -566,10 +547,25 @@
 ;;; fasl file header.)
 
 ;; Cold-load calls COLD-LOAD-CODE instead
-(!define-fop #xE0 :not-host (fop-code ((:operands n-code-bytes n-boxed-words nfuns)))
-  ;; add 1 word for the toplevel-p flag and one for the debug-info
-  (with-fop-stack ((stack (operand-stack)) ptr (+ n-boxed-words 2))
-    (load-code nfuns n-boxed-words n-code-bytes stack ptr (fasl-input))))
+(!define-fop #xE0 :not-host (fop-load-code ((:operands n-code-bytes n-boxed-words)))
+  (let ((n-constants (- n-boxed-words sb!vm:code-constants-offset)))
+    ;; stack has (at least) N-CONSTANTS words plus 2 more: toplevel-p and debug-info
+    (with-fop-stack ((stack (operand-stack)) ptr (+ n-constants 2))
+      (let* ((debug-info-index (+ ptr n-constants))
+             (immobile-p (svref stack (1+ debug-info-index)))
+             (n-boxed-words (+ sb!vm:code-constants-offset n-constants))
+             (code (sb!c:allocate-code-object
+                    immobile-p
+                    (align-up n-boxed-words sb!c::code-boxed-words-align)
+                    n-code-bytes)))
+        (setf (%code-debug-info code) (svref stack debug-info-index))
+        (loop for i of-type index from sb!vm:code-constants-offset
+              for j of-type index from ptr below debug-info-index
+              do (setf (code-header-ref code i) (svref stack j)))
+        (with-pinned-objects (code)
+          (read-n-bytes (fasl-input-stream) (code-instructions code) 0 n-code-bytes)
+          (sb!c::apply-fasl-fixups stack code))
+        code))))
 
 ;; this gets you an #<fdefn> object, not the result of (FDEFINITION x)
 ;; cold-loader uses COLD-FDEFINITION-OBJECT instead.
@@ -582,11 +578,6 @@
 (!define-fop 65 :not-host (fop-known-fun (name))
   (%coerce-name-to-fun name))
 
-#!-(or x86 (and x86-64 (not immobile-space)))
-(!define-fop 61 :not-host (fop-sanctify-for-execution (component))
-  (sb!vm:sanctify-for-execution component)
-  component)
-
 ;;; Modify a slot in a CONSTANTS object.
 (!define-fop 140 :not-host (fop-alter-code ((:operands index) code value) nil)
   (setf (code-header-ref code index) value)
@@ -618,84 +609,6 @@
 (!define-fop 144 (fop-assembler-code)
   (error "cannot load assembler code except at cold load"))
 
-(!define-fop 145 (fop-assembler-routine)
-  (error "cannot load assembler code except at cold load"))
-
-(!define-fop 146 :not-host (fop-symbol-tls-fixup (code-object kind symbol))
-  (sb!vm:fixup-code-object code-object
-                           (read-word-arg (fasl-input-stream))
-                           (ensure-symbol-tls-index symbol)
-                           kind)
-  code-object)
-
-#!+immobile-space
-(progn
-  (!define-fop 133 :not-host (fop-layout-fixup (code-object kind obj))
-    (sb!vm:fixup-code-object code-object
-                             (read-word-arg (fasl-input-stream))
-                             (get-lisp-obj-address (find-layout obj))
-                             kind :immobile-object)
-    code-object)
-  (!define-fop 134 :not-host (fop-immobile-obj-fixup (code-object kind obj))
-    (sb!vm:fixup-code-object code-object
-                             (read-word-arg (fasl-input-stream))
-                             (get-lisp-obj-address obj) ; OBJ can't move
-                             kind :immobile-object)
-    code-object))
-
-#!+immobile-code
-(progn
-  (!define-fop 135 :not-host (fop-named-call-fixup (code-object kind name))
-    (sb!vm:fixup-code-object code-object
-                             (read-word-arg (fasl-input-stream))
-                             (sb!vm::fdefn-entry-address name)
-                             kind :named-call)
-    code-object)
-
-  (!define-fop 136 :not-host (fop-static-call-fixup (code-object kind name))
-    (sb!vm:fixup-code-object code-object
-                             (read-word-arg (fasl-input-stream))
-                             (sb!vm::function-raw-address name)
-                             kind :static-call)
-    code-object))
-
-(!define-fop 147 :not-host (fop-foreign-fixup (code-object kind))
-  (let* ((len (read-byte-arg (fasl-input-stream)))
-         (sym (make-string len :element-type 'base-char)))
-    (read-n-bytes (fasl-input-stream) sym 0 len)
-    (sb!vm:fixup-code-object code-object
-                             (read-word-arg (fasl-input-stream))
-                             (foreign-symbol-address sym)
-                             kind)
-    code-object))
-
-(!define-fop 148 :not-host (fop-assembler-fixup (code-object kind routine))
-  (multiple-value-bind (value found) (gethash routine *assembler-routines*)
-    (unless found
-      (error "undefined assembler routine: ~S" routine))
-    (sb!vm:fixup-code-object code-object (read-word-arg (fasl-input-stream))
-                             value kind :assembly-routine))
-  code-object)
-
-(!define-fop 149 :not-host (fop-code-object-fixup (code-object kind))
-    ;; Note: We don't have to worry about GC moving the code-object after
-    ;; the GET-LISP-OBJ-ADDRESS and before that value is deposited, because
-    ;; we can only use code-object fixups when code-objects don't move.
-  (sb!vm:fixup-code-object code-object (read-word-arg (fasl-input-stream))
-                           (get-lisp-obj-address code-object) kind)
-  code-object)
-
-#!+linkage-table
-(!define-fop 150 :not-host (fop-foreign-dataref-fixup (code-object kind))
-  (let* ((len (read-byte-arg (fasl-input-stream)))
-         (sym (make-string len :element-type 'base-char)))
-    (read-n-bytes (fasl-input-stream) sym 0 len)
-    (sb!vm:fixup-code-object code-object
-                             (read-word-arg (fasl-input-stream))
-                             (foreign-symbol-address sym t)
-                             kind)
-    code-object))
-
 ;;; FOPs needed for implementing an IF operator in a FASL
 
 ;;; Skip until a FOP-MAYBE-STOP-SKIPPING with the same POSITION is
