@@ -40,12 +40,21 @@
 
 #!+immobile-space
 (progn
+  (define-alien-variable sb!vm:varyobj-space-size (unsigned 32))
   #!+relocatable-heap
-  (define-alien-variable ("IMMOBILE_SPACE_START" sb!vm:immobile-space-start) unsigned-long)
+  (progn
+    (define-alien-variable ("FIXEDOBJ_SPACE_START" sb!vm:fixedobj-space-start) unsigned-long)
+    (define-alien-variable ("VARYOBJ_SPACE_START" sb!vm:varyobj-space-start) unsigned-long))
+  (define-alien-variable ("varyobj_free_pointer" sb!vm:*varyobj-space-free-pointer*)
+    system-area-pointer)
+  (define-alien-variable ("fixedobj_free_pointer" sb!vm:*fixedobj-space-free-pointer*)
+    system-area-pointer)
   (defun immobile-space-addr-p (addr)
     (declare (type word addr))
-    (let ((start sb!vm:immobile-space-start))
-      (<= start addr (truly-the word (+ start sb!vm:immobile-space-size)))))
+    (or (let ((start sb!vm:fixedobj-space-start))
+          (<= start addr (truly-the word (+ start (1- sb!vm:fixedobj-space-size)))))
+        (let ((start sb!vm:varyobj-space-start))
+          (<= start addr (truly-the word (+ start (1- sb!vm:varyobj-space-size)))))))
   (defun immobile-space-obj-p (obj)
     (immobile-space-addr-p (get-lisp-obj-address obj))))
 
@@ -78,7 +87,7 @@
 ;;; ever allocated by adding this to the number of bytes currently
 ;;; allocated and never freed.)
 (declaim (type unsigned-byte *n-bytes-freed-or-purified*))
-(defglobal *n-bytes-freed-or-purified* 0)
+(define-load-time-global *n-bytes-freed-or-purified* 0)
 (defun gc-reinit ()
   (setq *gc-inhibit* nil)
   (gc)
@@ -109,11 +118,11 @@ run in any thread.")
 (define-alien-routine collect-garbage int
   (#!+gencgc last-gen #!-gencgc ignore int))
 
-#!+sb-thread
+#!+(or sb-thread sb-safepoint)
 (progn
   (define-alien-routine gc-stop-the-world void)
   (define-alien-routine gc-start-the-world void))
-#!-sb-thread
+#!-(or sb-thread sb-safepoint)
 (progn
   (defun gc-stop-the-world ())
   (defun gc-start-the-world ()))
@@ -188,12 +197,12 @@ statistics are appended to it."
                       ;; turn is a type-error.
                       (when (plusp run-time)
                         (incf *gc-run-time* run-time)))
-                    #!+sb-safepoint
+                    #!+(and sb-thread sb-safepoint)
                     (setf *stop-for-gc-pending* nil)
                     (setf *gc-pending* nil
                           new-usage (dynamic-usage))
                     #!+sb-thread
-                    (assert (not *stop-for-gc-pending*))
+                    (aver (not *stop-for-gc-pending*))
                     (gc-start-the-world)
                     ;; In a multithreaded environment the other threads
                     ;; will see *n-b-f-o-p* change a little late, but
@@ -214,7 +223,6 @@ statistics are appended to it."
              (sb!thread::without-thread-waiting-for
                  (:already-without-interrupts t)
                (let ((sb!impl::*deadline* nil)
-                     (sb!impl::*deadline-seconds* nil)
                      (epoch *gc-epoch*))
                  (loop
                   ;; GCing must be done without-gcing to avoid
@@ -264,7 +272,7 @@ statistics are appended to it."
   ;; another GC. FIXME: it can potentially exceed maximum interrupt
   ;; nesting by triggering GCs.
   ;;
-  ;; Can that be avoided by having the finalizers and hooks run only
+  ;; Can that be avoided by having the hooks run only
   ;; from the outermost SUB-GC? If the nested GCs happen in interrupt
   ;; handlers that's not enough.
   ;;
@@ -277,13 +285,26 @@ statistics are appended to it."
   ;;    don't want to run user code in such a case.
   ;;
   ;; The long-term solution will be to keep a separate thread for
-  ;; finalizers and after-gc hooks.
+  ;; after-gc hooks.
+  ;; Finalizers are in a separate thread (usually),
+  ;; but it's not permissible to invoke CONDITION-NOTIFY from a
+  ;; dying thread, so we still need the guard for that, but not
+  ;; the guard for whether interupts are enabled.
   (when (sb!thread:thread-alive-p sb!thread:*current-thread*)
-    (when *allow-with-interrupts*
-      (sb!thread::without-thread-waiting-for ()
-        (with-interrupts
-          (run-pending-finalizers)
-          (call-hooks "after-GC" *after-gc-hooks* :on-error :warn))))))
+    (let ((threadp #!+sb-thread (%instancep sb!impl::*finalizer-thread*)))
+      (when threadp
+        ;; It's OK to frob a condition variable regardless of
+        ;; *allow-with-interrupts*, and probably OK to start a thread.
+        ;; For consistency with the previous behavior, we delay finalization
+        ;; if there is no finalizer thread and interrupts are disabled.
+        ;; That's my excuse anyway, not having looked more in-depth.
+        (run-pending-finalizers))
+      (when *allow-with-interrupts*
+        (sb!thread::without-thread-waiting-for ()
+         (with-interrupts
+           (unless threadp
+             (run-pending-finalizers))
+           (call-hooks "after-GC" *after-gc-hooks* :on-error :warn)))))))
 
 ;;; This is the user-advertised garbage collection function.
 (defun gc (&key (full nil) (gen 0) &allow-other-keys)
@@ -376,10 +397,6 @@ Note: currently changes to this value are lost when saving core."
 #!+gencgc
 (define-alien-type generation
     (struct generation
-            (alloc-large-start-page page-index-t)
-            (alloc-start-page page-index-t)
-            (alloc-unboxed-start-page page-index-t)
-            #!+segregated-code (alloc-code-start-page page-index-t)
             (bytes-allocated os-vm-size-t)
             (gc-trigger os-vm-size-t)
             (bytes-consed-between-gcs os-vm-size-t)
@@ -466,3 +483,8 @@ Experimental: interface subject to change."
     (alien-funcall (extern-alien "generation_average_age"
                                  (function double generation-index-t))
                    generation))
+
+(declaim (inline sb!vm:is-lisp-pointer))
+(defun sb!vm:is-lisp-pointer (addr) ; Same as is_lisp_pointer() in C
+  #!-64-bit (oddp addr)
+  #!+64-bit (not (logtest (logxor addr 3) 3)))
