@@ -703,8 +703,8 @@ line break."
 
 ;;;; pprint-dispatch tables
 
-(defglobal *standard-pprint-dispatch-table* nil)
-(defglobal *initial-pprint-dispatch-table* nil)
+(define-load-time-global *standard-pprint-dispatch-table* nil)
+(define-load-time-global *initial-pprint-dispatch-table* nil)
 
 (defstruct (pprint-dispatch-entry
             (:constructor make-pprint-dispatch-entry (type priority fun test-fn))
@@ -749,7 +749,6 @@ line break."
 ;; This used to involve rewriting into a sexpr if CONS was involved,
 ;; since it was not an official specifier. But now it is.
 (defun compute-test-fn (ctype type-spec function)
-  (declare (special sb!c::*backend-type-predicates*))
   ;; Avoid compiling code for an existing structure predicate
   (or (and (eq (info :type :kind type-spec) :instance)
            (let ((layout (info :type :compiler-layout type-spec)))
@@ -891,11 +890,7 @@ line break."
          (output-ugly-object stream array))
         ((and *print-readably*
               (not (array-readably-printable-p array)))
-         (if *read-eval*
-             (if (vectorp array)
-                 (sb!impl::output-unreadable-vector-readably array stream)
-                 (sb!impl::output-unreadable-array-readably array stream))
-             (print-not-readable-error array stream)))
+         (sb!impl::output-unreadable-array-readably array stream))
         ((vectorp array)
          (pprint-vector stream array))
         (t
@@ -1124,8 +1119,7 @@ line break."
        (write-char #\space stream)
        (pprint-newline :mandatory stream)))))
 
-(eval-when (:compile-toplevel :execute)
-(sb!xc:defmacro pprint-tagbody-guts (stream)
+(defmacro pprint-tagbody-guts (stream)
   `(loop
      (pprint-exit-if-list-exhausted)
      (write-char #\space ,stream)
@@ -1134,7 +1128,7 @@ line break."
                       (if (atom form-or-tag) 0 1)
                       ,stream)
        (pprint-newline :linear ,stream)
-       (output-object form-or-tag ,stream)))))
+       (output-object form-or-tag ,stream))))
 
 (defun pprint-tagbody (stream list &rest noise)
   (declare (ignore noise))
@@ -1235,7 +1229,7 @@ line break."
 ;;;        puts a newline in between INTO and COUNT.
 ;;;        It would be awesome to have code in common with the macro
 ;;;        the properly represents each clauses.
-(defglobal *loop-separating-clauses*
+(defconstant-eqx +loop-separating-clauses+
   '(:and
     :with :for
     :initially :finally
@@ -1249,7 +1243,8 @@ line break."
     :minimize :minimizing
     :if :when :unless :end
     :for :while :until :repeat :always :never :thereis
-    ))
+    )
+  #'equal)
 
 (defun pprint-extended-loop (stream list)
   (pprint-logical-block (stream list :prefix "(" :suffix ")")
@@ -1262,7 +1257,7 @@ line break."
     (write-char #\space stream)
     (loop for thing = (pprint-pop)
           when (and (symbolp thing)
-                    (member thing  *loop-separating-clauses* :test #'string=))
+                    (member thing +loop-separating-clauses+ :test #'string=))
           do (pprint-newline :mandatory stream)
           do (output-object thing stream)
           do (pprint-exit-if-list-exhausted)
@@ -1348,8 +1343,7 @@ line break."
 
 ;;;; the interface seen by regular (ugly) printer and initialization routines
 
-(eval-when (:compile-toplevel :execute)
-(sb!xc:defmacro with-pretty-stream ((stream-var
+(defmacro with-pretty-stream ((stream-var
                                      &optional (stream-expression stream-var))
                                     &body body)
   (let ((flet-name (sb!xc:gensym "WITH-PRETTY-STREAM")))
@@ -1362,7 +1356,7 @@ line break."
                (let ((stream (make-pretty-stream stream)))
                  (,flet-name stream)
                  (force-pretty-output stream)))))
-       nil))))
+       nil)))
 
 (defun call-logical-block-printer (proc stream prefix per-line-p suffix
                                    &optional (object nil obj-supplied-p))
