@@ -11,8 +11,8 @@
 
 (in-package "SB!VM")
 
-(defconstant arg-count-sc (make-sc-offset immediate-arg-scn nargs-offset))
-(defconstant closure-sc (make-sc-offset descriptor-reg-sc-number lexenv-offset))
+(defconstant arg-count-sc (make-sc+offset immediate-arg-scn nargs-offset))
+(defconstant closure-sc (make-sc+offset descriptor-reg-sc-number lexenv-offset))
 
 ;;; Make a passing location TN for a local call return PC.  If
 ;;; standard is true, then use the standard (full call) location,
@@ -25,7 +25,7 @@
                  lra-save-offset))
 
 (defconstant return-pc-passing-offset
-  (make-sc-offset control-stack-sc-number lra-save-offset))
+  (make-sc+offset control-stack-sc-number lra-save-offset))
 
 ;;; This is similar to MAKE-RETURN-PC-PASSING-LOCATION, but makes a
 ;;; location to pass OLD-FP in.
@@ -34,13 +34,12 @@
 ;;; because we want to be able to assume it's always there. Besides,
 ;;; the ARM doesn't have enough registers to really make it profitable
 ;;; to pass it in a register.
-(defun make-old-fp-passing-location (standard)
-  (declare (ignore standard))
+(defun make-old-fp-passing-location ()
   (make-wired-tn *fixnum-primitive-type* control-stack-sc-number
                  ocfp-save-offset))
 
 (defconstant old-fp-passing-offset
-  (make-sc-offset control-stack-sc-number ocfp-save-offset))
+  (make-sc+offset control-stack-sc-number ocfp-save-offset))
 
 ;;; Make the TNs used to hold OLD-FP and RETURN-PC within the current
 ;;; function. We treat these specially so that the debugger can find
@@ -114,7 +113,6 @@
 
 (define-vop (xep-allocate-frame)
   (:info start-lab)
-  (:temporary (:scs (non-descriptor-reg)) temp)
   (:temporary (:scs (interior-reg)) lip)
   (:generator 1
     ;; Make sure the function is aligned, and drop a label pointing to this
@@ -123,9 +121,8 @@
     (emit-label start-lab)
     ;; Allocate function header.
     (inst simple-fun-header-word)
-    (dotimes (i (1- simple-fun-code-offset))
-      (inst dword 0))
-    (inst compute-code code-tn lip start-lab temp)))
+    (inst .skip (* (1- simple-fun-code-offset) n-word-bytes))
+    (inst compute-code code-tn lip start-lab)))
 
 (define-vop (xep-setup-sp)
   (:vop-var vop)
@@ -193,15 +190,14 @@
 ;;;  -- Reset SP.  This must be done whenever other than 1 value is returned,
 ;;;     regardless of the number of values desired.
 
-(defun default-unknown-values (vop values nvals move-temp temp lip lra-label)
+(defun default-unknown-values (vop values nvals move-temp lip lra-label)
   (declare (type (or tn-ref null) values)
-           (type unsigned-byte nvals) (type tn move-temp temp))
-  (let ((expecting-values-on-stack (> nvals register-arg-count))
-        (values-on-stack temp))
+           (type unsigned-byte nvals) (type tn move-temp))
+  (let ((expecting-values-on-stack (> nvals register-arg-count)))
     (note-this-location vop (if (<= nvals 1)
                                 :single-value-return
                                 :unknown-return))
-    (inst compute-code code-tn lip lra-label temp)
+    (inst compute-code code-tn lip lra-label)
     ;; Pick off the single-value case first.
     (sb!assem:without-scheduling ()
 
@@ -212,7 +208,8 @@
         (do ((i 1 (1+ i))
              (val (tn-ref-across values) (tn-ref-across val)))
             ((= i (min nvals register-arg-count)))
-          (inst csel (tn-ref-tn val) null-tn (tn-ref-tn val) :ne)))
+          (unless (eq (tn-kind (tn-ref-tn val)) :unused)
+           (inst csel (tn-ref-tn val) null-tn (tn-ref-tn val) :ne))))
 
       ;; If we're not expecting values on the stack, all that
       ;; remains is to clear the stack frame (for the multiple-
@@ -232,26 +229,31 @@
         (inst mov tmp-tn (fixnumize 1))
         (inst csel nargs-tn tmp-tn nargs-tn :ne)
 
-        ;; Compute the number of stack values (may be negative if
-        ;; not all of the register values are populated).
-        (inst sub values-on-stack nargs-tn (fixnumize register-arg-count))
-
         ;; For each expected stack value...
         (do ((i register-arg-count (1+ i))
+             (decrement (fixnumize (1+ register-arg-count)))
              (val (do ((i 0 (1+ i))
                        (val values (tn-ref-across val)))
                       ((= i register-arg-count) val))
                   (tn-ref-across val)))
             ((null val))
-          (assemble ()
-            ;; ... Load it if there is a stack value available, or
-            ;; default it if there isn't.
-            (inst subs values-on-stack values-on-stack (fixnumize 1))
-            (inst b :lt NONE)
-            (loadw move-temp ocfp-tn i 0)
-            NONE
-            (inst csel move-temp null-tn move-temp :lt)
-            (store-stack-tn (tn-ref-tn val) move-temp)))
+          (let ((tn (tn-ref-tn val)))
+            (if (eq (tn-kind tn) :unused)
+                (incf decrement (fixnumize 1))
+                (assemble ()
+                  ;; ... Load it if there is a stack value available, or
+                  ;; default it if there isn't.
+                  (inst subs nargs-tn nargs-tn decrement)
+                  (setf decrement (fixnumize 1))
+                  (inst b :lt NONE)
+                  (loadw move-temp ocfp-tn i 0)
+                  NONE
+                  (sc-case tn
+                    (control-stack
+                     (inst csel move-temp null-tn move-temp :lt)
+                     (store-stack-tn tn move-temp))
+                    (t
+                     (inst csel tn null-tn move-temp :lt)))))))
         ;; Deallocate the callee stack frame.
         (move csp-tn ocfp-tn))))
   (values))
@@ -274,10 +276,10 @@
 ;;;    Args and Nargs are TNs wired to the named locations.  We must
 ;;; explicitly allocate these TNs, since their lifetimes overlap with the
 ;;; results Start and Count (also, it's nice to be able to target them).
-(defun receive-unknown-values (args nargs start count lra-label temp lip)
-  (declare (type tn args nargs start count temp))
+(defun receive-unknown-values (args nargs start count lra-label lip)
+  (declare (type tn args nargs start count))
   (assemble ()
-    (inst compute-code code-tn lip lra-label temp)
+    (inst compute-code code-tn lip lra-label)
     (inst b :eq MULTIPLE)
     (move start csp-tn)
     (inst add csp-tn csp-tn n-word-bytes)
@@ -285,27 +287,26 @@
     (inst mov count (fixnumize 1))
     (inst b DONE)
     MULTIPLE
-    (do ((arg *register-arg-tns* (rest arg))
-         (i 0 (1+ i)))
+    #.(assert (evenp register-arg-count))
+    (do ((arg *register-arg-tns* (cddr arg))
+         (i 0 (+ i 2)))
         ((null arg))
-      (storew (first arg) args i 0))
+      (inst stp (first arg) (second arg)
+            (@ args (* i n-word-bytes))))
     (move start args)
     (move count nargs)
     DONE))
 
-
 ;;; VOP that can be inherited by unknown values receivers.  The main
 ;;; thing this handles is allocation of the result temporaries.
 (define-vop (unknown-values-receiver)
   (:results
-   (start :scs (any-reg) :from (:save 0))
-   (count :scs (any-reg) :from (:save 1)))
-  (:temporary (:sc any-reg :offset ocfp-offset
-               :from :eval :to :save)
-              values-start)
-  (:temporary (:sc any-reg :offset nargs-offset
-               :from :eval :to :save)
-              nvals))
+   (start :scs (any-reg))
+   (count :scs (any-reg)))
+  (:temporary (:sc any-reg :offset ocfp-offset :from :result) values-start)
+  (:temporary (:sc any-reg :offset nargs-offset :from :result) nvals)
+  ;; Avoid being clobbered by RECEIVE-UNKNOWN-VALUES
+  (:temporary (:sc descriptor-reg :offset r0-offset :from :result) r0-temp))
 
 ;;; This hook in the codegen pass lets us insert code before fall-thru entry
 ;;; points, local-call entry points, and tail-call entry points.  The default
@@ -365,7 +366,8 @@
     ;; actually setting up the "real" NFP.
     (let ((result (make-random-tn :kind :normal
                                   :sc (sc-or-lose 'any-reg)
-                                  :offset nfp-offset)))
+                                  :offset nfp-offset))
+          (delta (- (sb-allocated-size 'control-stack) fixed)))
       ;; And we use ASSEMBLE here so that we get "implcit labels"
       ;; rather than having to use GEN-LABEL and EMIT-LABEL.
       (assemble ()
@@ -384,7 +386,9 @@
                (inst csel csp-tn result csp-tn :le)
                (inst b :le DONE)
                (inst add dest result (lsl count (- word-shift n-fixnum-tag-bits)))
-               (move csp-tn dest)))
+               ;; Don't leave the arguments unprotected when moving below the stack pointer
+               (when (>= delta 0)
+                 (move csp-tn dest))))
 
         (when (< fixed register-arg-count)
           ;; We must stop when we run out of stack args, not when we
@@ -403,22 +407,24 @@
         ;; moved.
 
         LOOP
-        (let ((delta (- (sb-allocated-size 'control-stack) fixed)))
-          (cond ((zerop delta)) ;; nothing to move
-                ((plusp delta)  ;; copy backward
+        (cond ((zerop delta)) ;; nothing to move
+              ((plusp delta)   ;; copy backward
+               (inst cmp dest result)
+               (inst b :le DO-REGS)
+               (inst ldr temp (@ dest (load-store-offset
+                                       (- (* (1+ delta) n-word-bytes)))))
+               (inst str temp (@ dest (- n-word-bytes) :pre-index))
+               (inst b LOOP))
+              (t ;; copy forward
+               (assemble ()
                  (inst cmp dest result)
-                 (inst b :le DO-REGS)
-                 (inst ldr temp (@ dest (load-store-offset
-                                         (- (* (1+ delta) n-word-bytes)))))
-                 (inst str temp (@ dest (- n-word-bytes) :pre-index))
-                 (inst b LOOP))
-                (t ;; copy forward
-                 (inst cmp dest result)
-                 (inst b :le DO-REGS)
+                 (inst b :le DONE)
                  (inst ldr temp (@ result (load-store-offset
                                            (- (* delta n-word-bytes)))))
                  (inst str temp (@ result n-word-bytes :post-index))
-                 (inst b LOOP))))
+                 (inst b LOOP)
+                 DONE
+                 (inst mov csp-tn dest))))
 
         DO-REGS
         (when (< fixed register-arg-count)
@@ -494,8 +500,7 @@
 
       ;; Grab one value.
       ENTER
-      (loadw temp context)
-      (inst add context context n-word-bytes)
+      (inst ldr temp (@ context n-word-bytes :post-index))
 
       ;; Dec count, and if != zero, go back for more.
       (inst subs count count (fixnumize 1))
@@ -541,24 +546,25 @@
   (:generator 3
     (let ((err-lab
            (generate-error-code vop 'invalid-arg-count-error)))
-      (flet ((load-immediate (x)
-               (add-sub-immediate (fixnumize x))))
+      (labels ((load-immediate (x)
+                 (add-sub-immediate (fixnumize x)))
+               (check-min ()
+                 (cond ((= min 1)
+                        (inst cbz nargs err-lab))
+                       ((plusp min)
+                        (inst cmp nargs (load-immediate min))
+                        (inst b :lo err-lab)))))
         (cond ((eql max 0)
                (inst cbnz nargs err-lab))
               ((not min)
                (inst cmp nargs (load-immediate max))
                (inst b :ne err-lab))
-             (max
-              (when (plusp min)
-                (inst cmp nargs (load-immediate min))
-                (inst b :lo err-lab))
-              (inst cmp nargs (load-immediate max))
-              (inst b :hi err-lab))
-             ((eql min 1)
-              (inst cbz nargs err-lab))
-             ((plusp min)
-              (inst cmp nargs (load-immediate min))
-              (inst b :lo err-lab)))))))
+              (max
+               (check-min)
+               (inst cmp nargs (load-immediate max))
+               (inst b :hi err-lab))
+              (t
+               (check-min)))))))
 
 ;;;; Local call with unknown values convention return:
 
@@ -589,7 +595,6 @@
   (:info arg-locs callee target nvals)
   (:vop-var vop)
   (:temporary (:scs (descriptor-reg) :from (:eval 0)) move-temp)
-  (:temporary (:scs (non-descriptor-reg)) temp)
   (:temporary (:sc control-stack :offset nfp-save-offset) nfp-save)
   (:temporary (:sc any-reg :offset ocfp-offset :from (:eval 0)) ocfp)
   (:temporary (:scs (interior-reg)) lip)
@@ -608,9 +613,7 @@
       (note-this-location vop :call-site)
       (inst b target)
       (emit-return-pc label)
-      (default-unknown-values vop values nvals move-temp temp lip label)
-      ;; alpha uses (maybe-load-stack-nfp-tn cur-nfp nfp-save temp)
-      ;; instead of the clause below
+      (default-unknown-values vop values nvals move-temp lip label)
       (when cur-nfp
         (load-stack-tn cur-nfp nfp-save)))))
 
@@ -629,10 +632,9 @@
   (:save-p t)
   (:move-args :local-call)
   (:info save callee target)
-  (:ignore args save)
+  (:ignore args save r0-temp)
   (:vop-var vop)
   (:temporary (:sc control-stack :offset nfp-save-offset) nfp-save)
-  (:temporary (:scs (non-descriptor-reg)) temp)
   (:temporary (:scs (interior-reg)) lip)
   (:generator 20
     (let ((label (gen-label))
@@ -650,7 +652,7 @@
       (inst b target)
       (emit-return-pc label)
       (note-this-location vop :unknown-return)
-      (receive-unknown-values values-start nvals start count label temp lip)
+      (receive-unknown-values values-start nvals start count label lip)
       (when cur-nfp
         (load-stack-tn cur-nfp nfp-save)))))
 
@@ -770,9 +772,12 @@
       ,@(unless (eq return :tail)
           '((new-fp :scs (any-reg) :to :eval)))
 
-      ,(if named
-           '(name :target name-pass)
-           '(arg-fun :target lexenv))
+      ,@(case named
+         ((nil)
+          '((arg-fun :target lexenv)))
+         (:direct)
+         (t
+          '((name :target name-pass))))
 
       ,@(when (eq return :tail)
           '((old-fp)
@@ -791,23 +796,30 @@
      (:vop-var vop)
      (:info ,@(unless (or variable (eq return :tail)) '(arg-locs))
             ,@(unless variable '(nargs))
+            ,@(when (eq named :direct) '(fun))
             ,@(when (eq return :fixed) '(nvals))
             step-instrumenting)
 
      (:ignore
-      ,@(when (eq return :fixed) '(ocfp-temp))
       ,@(unless (or variable (eq return :tail)) '(arg-locs))
       ,@(unless variable '(args))
-      ,@(when (eq return :tail) '(old-fp)))
+      ,(ecase return
+         (:fixed 'ocfp-temp)
+         (:tail 'old-fp)
+         (:unknown 'r0-temp)))
 
-     (:temporary (:sc descriptor-reg :offset lexenv-offset
-                      :from (:argument ,(if (eq return :tail) 0 1))
-                      :to :eval)
-                 ,(if named 'name-pass 'lexenv))
+     ,@(unless (eq named :direct)
+         `((:temporary (:sc descriptor-reg :offset lexenv-offset
+                        :from (:argument ,(if (eq return :tail) 0 1))
+                        :to :eval)
+                       ,(if named 'name-pass 'lexenv))
+           (:temporary (:scs (descriptor-reg) :to :eval)
+                       function)))
 
-     (:temporary (:scs (descriptor-reg) :to :eval)
-               function)
-     (:temporary (:sc any-reg :offset nargs-offset :to :eval)
+     (:temporary (:sc any-reg :offset nargs-offset :to
+                      ,(if (eq return :fixed)
+                           :save
+                           :eval))
                  nargs-pass)
 
      ,@(when variable
@@ -822,8 +834,7 @@
            (:temporary (:sc any-reg :from :eval :offset ocfp-offset) ocfp-temp)))
 
      ,@(unless (eq return :tail)
-         '((:temporary (:scs (non-descriptor-reg)) temp)
-           (:temporary (:sc control-stack :offset nfp-save-offset) nfp-save)))
+         '((:temporary (:sc control-stack :offset nfp-save-offset) nfp-save)))
 
      (:temporary (:scs (interior-reg)) lip)
 
@@ -867,10 +878,14 @@
                                (inst add csp-tn nargs-pass (* 2 n-word-bytes))
                                (inst sub nargs-pass nargs-pass new-fp)
                                (inst asr nargs-pass nargs-pass (- word-shift n-fixnum-tag-bits))
-                               ,@(let ((index -1))
-                                   (mapcar #'(lambda (name)
-                                               `(loadw ,name new-fp ,(incf index)))
-                                           *register-arg-names*))
+                               ,@(do ((arg *register-arg-names* (cddr arg))
+                                      (i 0 (+ i 2))
+                                      (insts))
+                                     ((null arg) (nreverse insts))
+                                   #.(assert (evenp register-arg-count))
+                                   (push `(inst ldp ,(first arg) ,(second arg)
+                                                (@ new-fp ,(* i n-word-bytes)))
+                                         insts))
                                (storew cfp-tn new-fp ocfp-save-offset))
                              '((inst mov nargs-pass (fixnumize nargs)))))
                       ,@(if (eq return :tail)
@@ -889,7 +904,7 @@
                               (:load-fp
                                (move cfp-tn new-fp))))
                       ((nil)))))
-                (insert-step-instrumenting (callable-tn)
+                (insert-step-instrumenting ()
                   ;; Conditionally insert a conditional trap:
                   (when step-instrumenting
                     (assemble ()
@@ -902,54 +917,52 @@
                       ;; interrupt is handled, not after the
                       ;; DEBUG-TRAP.
                       (note-this-location vop :step-before-vop)
-                      ;; Best-guess at a usable trap.  x86oids don't
-                      ;; have much more than this, SPARC, MIPS, PPC
-                      ;; and HPPA encode (TN-OFFSET CALLABLE-TN),
-                      ;; Alpha ignores stepping entirely.
                       (inst brk single-step-around-trap)
-                      (inst byte (tn-offset callable-tn))
-                      (emit-alignment 2)
-
                       STEP-DONE-LABEL))))
-
-
-           ,@(if named
-                 `((sc-case name
-                     (descriptor-reg (move name-pass name))
-                     (control-stack
-                      (load-stack-tn name-pass name)
-                      (do-next-filler))
-                     (constant
-                      (load-constant vop name name-pass)
-                      (do-next-filler)))
-                   (do-next-filler)
-                   (insert-step-instrumenting name-pass))
-                 `((sc-case arg-fun
-                     (descriptor-reg (move lexenv arg-fun))
-                     (control-stack
-                      (load-stack-tn lexenv arg-fun)
-                      (do-next-filler))
-                     (constant
-                      (load-constant vop arg-fun lexenv)
-                      (do-next-filler)))
-                   (loadw function lexenv closure-fun-slot
-                          fun-pointer-lowtag)
-                   (do-next-filler)
-                   (insert-step-instrumenting lip)))
+           (declare (ignorable #'insert-step-instrumenting))
+           ,@(case named
+               ((t)
+                `((sc-case name
+                    (descriptor-reg (move name-pass name))
+                    (control-stack
+                     (load-stack-tn name-pass name)
+                     (do-next-filler))
+                    (constant
+                     (load-constant vop name name-pass)
+                     (do-next-filler)))
+                  (do-next-filler)
+                  (insert-step-instrumenting)))
+               ((nil)
+                `((sc-case arg-fun
+                    (descriptor-reg (move lexenv arg-fun))
+                    (control-stack
+                     (load-stack-tn lexenv arg-fun)
+                     (do-next-filler))
+                    (constant
+                     (load-constant vop arg-fun lexenv)
+                     (do-next-filler)))
+                  (insert-step-instrumenting)
+                  (loadw function lexenv closure-fun-slot
+                      fun-pointer-lowtag)
+                  (do-next-filler))))
            (loop
              (if filler
                  (do-next-filler)
                  (return)))
-           ,@(if named
-                 ;; raw-addr is an untagged pointer to the function,
-                 ;; need to pair it up with the tagged pointer for the GC to see
-                 `((loadw function name-pass fdefn-fun-slot
-                       other-pointer-lowtag)
-                   (loadw lip name-pass fdefn-raw-addr-slot
-                       other-pointer-lowtag))
-                 `((inst add lip function
-                         (- (ash simple-fun-code-offset word-shift)
-                            fun-pointer-lowtag))))
+           ,@(ecase named
+               ;; raw-addr is an untagged pointer to the function,
+               ;; need to pair it up with the tagged pointer for the GC to see
+               ((t)
+                `((loadw function name-pass fdefn-fun-slot
+                      other-pointer-lowtag)
+                  (loadw lip name-pass fdefn-raw-addr-slot
+                      other-pointer-lowtag)))
+               (:direct
+                `((inst ldr lip (@ null-tn (load-store-offset (static-fun-offset fun))))))
+               ((nil)
+                `((inst add lip function
+                        (- (ash simple-fun-code-offset word-shift)
+                           fun-pointer-lowtag)))))
 
            (note-this-location vop :call-site)
            (inst br lip))
@@ -957,25 +970,27 @@
          ,@(ecase return
              (:fixed
               '((emit-return-pc lra-label)
-                (default-unknown-values vop values nvals move-temp
-                                        temp lip lra-label)
+                (default-unknown-values vop values nvals move-temp lip lra-label)
                 (when cur-nfp
                   (load-stack-tn cur-nfp nfp-save))))
              (:unknown
               '((emit-return-pc lra-label)
                 (note-this-location vop :unknown-return)
                 (receive-unknown-values values-start nvals start count
-                                        lra-label temp lip)
+                                        lra-label lip)
                 (when cur-nfp
                   (load-stack-tn cur-nfp nfp-save))))
              (:tail))))))
 
 (define-full-call call nil :fixed nil)
 (define-full-call call-named t :fixed nil)
+(define-full-call static-call-named :direct :fixed nil)
 (define-full-call multiple-call nil :unknown nil)
 (define-full-call multiple-call-named t :unknown nil)
+(define-full-call static-multiple-call-named :direct :unknown nil)
 (define-full-call tail-call nil :tail nil)
 (define-full-call tail-call-named t :tail nil)
+(define-full-call static-tail-call-named :direct :tail nil)
 
 (define-full-call call-variable nil :fixed t)
 (define-full-call multiple-call-variable nil :unknown t)
