@@ -124,7 +124,7 @@
        new-value
        (array-type-specialized-element-type type)
        (lexenv-policy (node-lexenv (lvar-dest new-value)))
-       'aref)))
+       :aref)))
   (lvar-type new-value))
 
 ;;; Return true if ARG is NIL, or is a constant-lvar whose
@@ -152,10 +152,10 @@
   (block nil
     (flet ((give-up (&optional reason)
              (cond ((= (length subscripts) 1)
-                    (let ((arg (sb!xc:gensym)))
-                      `(lambda (array ,arg)
-                         (and (typep ,arg '(and fixnum unsigned-byte))
-                              (< ,arg (array-dimension array 0))))))
+                    (return
+                      `(lambda (array arg)
+                         (and (typep arg '(and fixnum unsigned-byte))
+                              (< arg (array-dimension array 0))))))
                    (t
                     (give-up-ir1-transform
                      (or reason
@@ -168,9 +168,6 @@
                               (lvar-conservative-type array))
                              args)
                           (give-up (car args)))))
-        ;; Might be *. (Note: currently this is never true, because the type
-        ;; derivation infers the rank from the call to ARRAY-IN-BOUNDS-P, but
-        ;; let's keep this future proof.)
         (when (eq '* dimensions)
           (give-up "array bounds unknown"))
         ;; shortcut for zero dimensions
@@ -286,8 +283,19 @@
   (declare (ignore index))
   (assert-new-value-type new-value array))
 
+(defun check-array-dimensions (dims node)
+  (or (typep dims 'index)
+      (and (proper-list-p dims)
+           (every (lambda (x)
+                    (typep x 'index))
+                  dims))
+      (let ((*compiler-error-context* node))
+        (setf (combination-kind node) :error)
+        (compiler-warn "Bad array dimensions: ~s" dims))))
+
 (defun derive-make-array-type (dims element-type adjustable
-                               fill-pointer displaced-to)
+                               fill-pointer displaced-to
+                               node)
   (let* ((simple (and (unsupplied-or-nil adjustable)
                       (unsupplied-or-nil displaced-to)
                       (unsupplied-or-nil fill-pointer)))
@@ -308,6 +316,8 @@
                  ,(cond ((constant-lvar-p dims)
                          (let* ((val (lvar-value dims))
                                 (cdims (ensure-list val)))
+                           (unless (check-array-dimensions val node)
+                             (return-from derive-make-array-type))
                            (if simple
                                cdims
                                (length cdims))))
@@ -326,13 +336,16 @@
 
 (defoptimizer (make-array derive-type)
     ((dims &key element-type adjustable fill-pointer displaced-to
-           &allow-other-keys))
+           &allow-other-keys)
+     node)
   (derive-make-array-type dims element-type adjustable
-                          fill-pointer displaced-to))
+                          fill-pointer displaced-to
+                          node))
 
 (defoptimizer (%make-array derive-type)
     ((dims widetag n-bits &key adjustable fill-pointer displaced-to
-                          &allow-other-keys))
+           &allow-other-keys)
+     node)
   (declare (ignore n-bits))
   (let ((saetp (and (constant-lvar-p widetag)
                     (find (lvar-value widetag)
@@ -341,7 +354,8 @@
     (derive-make-array-type dims (if saetp
                                      (sb!vm:saetp-ctype saetp)
                                      *wild-type*)
-                            adjustable fill-pointer displaced-to)))
+                            adjustable fill-pointer displaced-to
+                            node)))
 
 
 ;;;; constructors
@@ -353,8 +367,7 @@
 ;;; Just convert it into a MAKE-ARRAY.
 (deftransform make-string ((length &key
                                    (element-type 'character)
-                                   (initial-element
-                                    #.*default-init-char-form*)))
+                                   (initial-element (code-char 0))))
   `(the simple-string (make-array (the index length)
                        :element-type element-type
                        ,@(when initial-element
@@ -385,7 +398,9 @@
     (return-from rewrite-initial-contents (values nil nil)))
   (let ((dimensions (make-array rank :initial-element nil))
         (output))
-    (named-let recurse ((form (sb!xc:macroexpand initial-contents env))
+    (named-let recurse ((form (handler-case (sb!xc:macroexpand initial-contents env)
+                                (error ()
+                                  (return-from rewrite-initial-contents))))
                         (axis 0))
       (flet ((make-list-ctor (tail &optional (prefix nil prefixp) &aux val)
                (when (and (sb!xc:constantp tail)
@@ -455,11 +470,14 @@
 (define-source-transform make-array (dims-form &rest rest &environment env
                                                &aux dims dims-constp)
   (cond ((and (sb!xc:constantp dims-form env)
-              (listp (setq dims (constant-form-value dims-form env)))
+              (proper-list-p (setq dims (constant-form-value dims-form env)))
               (not (singleton-p dims))
               (every (lambda (x) (typep x 'index)) dims))
          (setq dims-constp t))
-        ((and (cond ((typep (setq dims (sb!xc:macroexpand dims-form env))
+        ((and (cond ((typep (setq dims (handler-case
+                                           (sb!xc:macroexpand dims-form env)
+                                         (error ()
+                                           (return-from make-array (values nil t)))))
                             '(cons (eql list)))
                      (setq dims (cdr dims))
                      t)
@@ -582,7 +600,9 @@
       (let* ((type (constant-form-value type env))
              (length (1- (length x)))
              (ctype (careful-values-specifier-type type)))
-        (if (csubtypep ctype (specifier-type '(array * (*))))
+        (if (and ctype
+                 (neq ctype *empty-type*)
+                 (csubtypep ctype (specifier-type '(array * (*)))))
             (multiple-value-bind (type element-type upgraded had-dimensions)
                 (simplify-vector-type ctype)
               (declare (ignore type upgraded))
@@ -596,6 +616,11 @@
             (values nil t)))
       (values nil t)))
 
+(defun proper-sequence-p (sequence)
+  (if (consp sequence)
+      (proper-list-p sequence)
+      (typep sequence 'sequence)))
+
 ;;; This baby is a bit of a monster, but it takes care of any MAKE-ARRAY
 ;;; call which creates a vector with a known element type -- and tries
 ;;; to do a good job with all the different ways it can happen.
@@ -628,32 +653,38 @@
                        (lvar-value element-type) ; enforces const-ness.
                        t))
          (elt-ctype (ir1-transform-specifier-type elt-spec))
-         (saetp (if (unknown-type-p elt-ctype)
-                    (give-up-ir1-transform "~S is an unknown type: ~S"
-                                           :element-type elt-spec)
-                    (find-saetp-by-ctype elt-ctype)))
+         (saetp (cond ((unknown-type-p elt-ctype)
+                       (give-up-ir1-transform "~S is an unknown type: ~S"
+                                              :element-type elt-spec))
+                      ((eq elt-ctype *empty-type*)
+                       (give-up-ir1-transform))
+                      (t
+                       (find-saetp-by-ctype elt-ctype))))
          (default-initial-element (sb!vm:saetp-initial-element-default saetp))
          (n-bits (sb!vm:saetp-n-bits saetp))
          (typecode (sb!vm:saetp-typecode saetp))
          (n-pad-elements (sb!vm:saetp-n-pad-elements saetp))
          (n-words-form
-           (if c-length
-               (ceiling (* (+ c-length n-pad-elements) n-bits)
-                        sb!vm:n-word-bits)
-               (let ((padded-length-form (if (zerop n-pad-elements)
-                                             'length
-                                             `(+ length ,n-pad-elements))))
-                 (cond
-                   ((= n-bits 0) 0)
-                   ((>= n-bits sb!vm:n-word-bits)
-                    `(* ,padded-length-form
-                        ;; i.e., not RATIO
-                        ,(the fixnum (/ n-bits sb!vm:n-word-bits))))
-                   (t
-                    (let ((n-elements-per-word (/ sb!vm:n-word-bits n-bits)))
-                      (declare (type index n-elements-per-word)) ; i.e., not RATIO
-                      `(ceiling (truly-the index ,padded-length-form)
-                                ,n-elements-per-word)))))))
+           (cond ((not c-length)
+                  (let ((padded-length-form (if (zerop n-pad-elements)
+                                                'length
+                                                `(+ length ,n-pad-elements))))
+                    (cond
+                      ((= n-bits 0) 0)
+                      ((>= n-bits sb!vm:n-word-bits)
+                       `(* ,padded-length-form
+                           ;; i.e., not RATIO
+                           ,(the fixnum (/ n-bits sb!vm:n-word-bits))))
+                      (t
+                       (let ((n-elements-per-word (/ sb!vm:n-word-bits n-bits)))
+                         (declare (type index n-elements-per-word)) ; i.e., not RATIO
+                         `(ceiling (truly-the index ,padded-length-form)
+                                   ,n-elements-per-word))))))
+                 ((fixnump c-length)
+                  (ceiling (* (+ c-length n-pad-elements) n-bits)
+                           sb!vm:n-word-bits))
+                 (t
+                  (give-up-ir1-transform))))
          (data-result-spec
            `(simple-array ,(sb!vm:saetp-specifier saetp) (,(or c-length '*))))
          (result-spec
@@ -662,9 +693,6 @@
                      (not simple-array))
                `(simple-array
                  ,(sb!vm:saetp-specifier saetp) (,(or c-length '*)))))
-         (header-form (and complex
-                           `(make-array-header ,(or (sb!vm:saetp-complex-typecode saetp)
-                                                    sb!vm:complex-vector-widetag) 1)))
          (data-alloc-form
            `(truly-the ,data-result-spec
                        (allocate-vector ,typecode
@@ -694,34 +722,42 @@
                     (let* ((constant-fill-pointer-p (constant-lvar-p fill-pointer))
                            (fill-pointer-value (and constant-fill-pointer-p
                                                     (lvar-value fill-pointer))))
-                      `(let* ((header ,header-form)
-                              (data ,data-alloc-form)
-                              (data ,(or data-wrapper 'data))
-                              (length (the index ,(or c-length 'length))))
-                         (setf (%array-fill-pointer header)
-                               ,(cond ((eq fill-pointer-value t)
-                                       'length)
-                                      (fill-pointer-value)
-                                      ((and fill-pointer
-                                            (not constant-fill-pointer-p))
-                                       `(cond ((or (eq fill-pointer t)
-                                                   (null fill-pointer))
-                                               length)
-                                              ((> fill-pointer length)
-                                               (error "Invalid fill-pointer ~a" fill-pointer))
-                                              (t
-                                               fill-pointer)))
-                                      (t
-                                       'length)))
-                         (setf (%array-fill-pointer-p header)
-                               ,(and fill-pointer
-                                     `(and fill-pointer t)))
-                         (setf (%array-available-elements header) length)
-                         (setf (%array-data header) data)
-                         (setf (%array-displaced-p header) nil)
-                         (setf (%array-displaced-from header) nil)
-                         (setf (%array-dimension header 0) length)
-                         (truly-the ,result-spec header))))
+                      `(let ((%length (the index ,(or c-length 'length))))
+                         (truly-the
+                          ,result-spec
+                          (make-array-header* ,(or (sb!vm:saetp-complex-typecode saetp)
+                                                   sb!vm:complex-vector-widetag)
+                                              ;; fill-pointer
+                                              ,(cond ((eq fill-pointer-value t)
+                                                      '%length)
+                                                     (fill-pointer-value)
+                                                     ((and fill-pointer
+                                                           (not constant-fill-pointer-p))
+                                                      `(cond ((or (eq fill-pointer t)
+                                                                  (null fill-pointer))
+                                                              %length)
+                                                             ((> fill-pointer %length)
+                                                              (error "Invalid fill-pointer ~a" fill-pointer))
+                                                             (t
+                                                              fill-pointer)))
+                                                     (t
+                                                      '%length))
+                                              ;; fill-pointer-p
+                                              ,(and fill-pointer
+                                                    `(and fill-pointer t))
+                                              ;; elements
+                                              %length
+                                              ;; data
+                                              (let ((data ,data-alloc-form))
+                                                ,(or data-wrapper 'data))
+                                              ;; displacement
+                                              0
+                                              ;; displaced-p
+                                              nil
+                                              ;; displaced-from
+                                              nil
+                                              ;; dimensions
+                                              %length)))))
                    (data-wrapper
                     (subst data-alloc-form 'data data-wrapper))
                    (t
@@ -763,7 +799,9 @@
                   ;; To make matters worse, the time grows superlinearly,
                   ;; and it's not entirely obvious that passing a constant array
                   ;; of 100x100 things is responsible for such an explosion.
-                  (<= (length (lvar-value initial-contents)) 1000))
+                  (let ((initial-contents (lvar-value initial-contents)))
+                    (and (proper-sequence-p initial-contents)
+                         (<= (length initial-contents) 1000))))
              (let ((contents (lvar-value initial-contents)))
                (unless (= c-length (length contents))
                  (abort-ir1-transform "~S has ~S elements, vector length is ~S."
@@ -982,12 +1020,11 @@
           (element-type-ctype (and (constant-lvar-p element-type)
                                    (ir1-transform-specifier-type
                                     (lvar-value element-type)))))
-      (when (contains-unknown-type-p element-type-ctype)
+      (when (or (contains-unknown-type-p element-type-ctype)
+                (not (proper-list-p dims)))
         (give-up-ir1-transform))
-      (unless (every (lambda (x) (typep x '(integer 0))) dims)
-        (give-up-ir1-transform
-         "The dimension list contains something other than an integer: ~S"
-         dims))
+      (unless (check-array-dimensions dims call)
+          (give-up-ir1-transform))
       (cond ((singleton-p dims)
              (transform-make-array-vector (car dims) element-type
                                           initial-element initial-contents call
@@ -1014,30 +1051,34 @@
                                      (lvar-value element-type)))
                                    (t '*))
                             ,(make-list rank :initial-element '*))))
-               `(let ((header (make-array-header ,(if complex
-                                                      sb!vm:complex-array-widetag
-                                                      sb!vm:simple-array-widetag)
-                                                 ,rank))
-                      (data (make-array ,total-size
-                                        ,@(when element-type
-                                            '(:element-type element-type))
-                                        ,@(when initial-element
-                                            '(:initial-element initial-element)))))
-                  ,@(when initial-contents
-                      ;; FIXME: This is could be open coded at least a bit too
-                      `((fill-data-vector data ',dims initial-contents)))
-                  (setf (%array-fill-pointer header) ,total-size)
-                  (setf (%array-fill-pointer-p header) nil)
-                  (setf (%array-available-elements header) ,total-size)
-                  (setf (%array-data header) data)
-                  (setf (%array-displaced-p header) nil)
-                  (setf (%array-displaced-from header) nil)
-                  ,@(let ((axis -1))
-                      (mapcar (lambda (dim)
-                                `(setf (%array-dimension header ,(incf axis))
-                                       ,dim))
-                              dims))
-                  (truly-the ,spec header))))))))
+               `(truly-the ,spec
+                           (make-array-header* ,(if complex
+                                                    sb!vm:complex-array-widetag
+                                                    sb!vm:simple-array-widetag)
+                                               ;; fill-pointer
+                                               ,total-size
+                                               ;; fill-pointer-p
+                                               nil
+                                               ;; elements
+                                               ,total-size
+                                               ;; data
+                                               (let ((data (make-array ,total-size
+                                                                       ,@(when element-type
+                                                                           '(:element-type element-type))
+                                                                       ,@(when initial-element
+                                                                           '(:initial-element initial-element)))))
+                                                 ,(if initial-contents
+                                                      ;; FIXME: This is could be open coded at least a bit too
+                                                      `(fill-data-vector data ',dims initial-contents)
+                                                      'data))
+                                               ;; displacement
+                                               0
+                                               ;; displaced-p
+                                               nil
+                                               ;; displaced-from
+                                               nil
+                                               ;; dimensions
+                                               ,@dims))))))))
 
 (deftransform make-array ((dims &key element-type initial-element initial-contents
                                      adjustable fill-pointer)
@@ -1097,44 +1138,55 @@
 ;;; maybe this is just too sloppy for actual type logic.  -- CSR,
 ;;; 2004-02-18
 (defun array-type-dimensions-or-give-up (type)
-  (labels ((maybe-array-type-dimensions (type)
-             (typecase type
-               (array-type
-                (array-type-dimensions type))
-               (union-type
-                (let* ((types (loop for type in (union-type-types type)
-                                    for dimensions = (maybe-array-type-dimensions type)
-                                    when (eq dimensions '*)
-                                    do
-                                    (return-from maybe-array-type-dimensions '*)
-                                    when dimensions
-                                    collect it))
-                       (result (car types))
-                       (length (length result))
-                       (complete-match t))
-                  (dolist (other (cdr types))
-                    (when (/= length (length other))
-                      (give-up-ir1-transform
-                       "~@<dimensions of arrays in union type ~S do not match~:@>"
-                       (type-specifier type)))
-                    (unless (equal result other)
-                      (setf complete-match nil)))
-                  (if complete-match
-                      result
-                      (make-list length :initial-element '*))))
-               (intersection-type
-                (let* ((types (remove nil (mapcar #'maybe-array-type-dimensions
-                                                  (intersection-type-types type))))
-                       (result (car types)))
-                  (dolist (other (cdr types) result)
-                    (unless (equal result other)
-                      (abort-ir1-transform
-                       "~@<dimensions of arrays in intersection type ~S do not match~:@>"
-                       (type-specifier type)))))))))
-    (or (maybe-array-type-dimensions type)
-        (give-up-ir1-transform
-         "~@<don't know how to extract array dimensions from type ~S~:@>"
-         (type-specifier type)))))
+  (let ((no-dimension '#:no))
+    (labels ((maybe-array-type-dimensions (type)
+               (typecase type
+                 (array-type
+                  (array-type-dimensions type))
+                 (union-type
+                  (let* ((types (loop for type in (union-type-types type)
+                                      for dimensions = (maybe-array-type-dimensions type)
+                                      when (eq dimensions no-dimension)
+                                      do (return-from maybe-array-type-dimensions no-dimension)
+                                      when (eq dimensions '*)
+                                      do
+                                      (return-from maybe-array-type-dimensions '*)
+                                      unless (eq dimensions no-dimension)
+                                      collect dimensions))
+                         (result (car types))
+                         (length (length result))
+                         (complete-match t))
+                    (dolist (other (cdr types))
+                      (when (/= length (length other))
+                        (give-up-ir1-transform
+                         "~@<dimensions of arrays in union type ~S do not match~:@>"
+                         (type-specifier type)))
+                      (unless (equal result other)
+                        (setf complete-match nil)))
+                    (if complete-match
+                        result
+                        (make-list length :initial-element '*))))
+                 (intersection-type
+                  (let* ((types (remove no-dimension
+                                        (mapcar #'maybe-array-type-dimensions
+                                                (intersection-type-types type))))
+                         (result (car types)))
+                    (dolist (other (cdr types))
+                      (unless (equal result other)
+                        (abort-ir1-transform
+                         "~@<dimensions of arrays in intersection type ~S do not match~:@>"
+                         (type-specifier type))))
+                    (if types
+                        result
+                        no-dimension)))
+                 (t
+                  no-dimension))))
+      (let ((dim (maybe-array-type-dimensions type)))
+        (if (eq dim no-dimension)
+            (give-up-ir1-transform
+             "~@<don't know how to extract array dimensions from type ~S~:@>"
+             (type-specifier type))
+            dim)))))
 
 (defun conservative-array-type-complexp (type)
   (typecase type
@@ -1517,7 +1569,7 @@
     (with-unique-names (n-vector)
       `(let ((,n-vector ,vector))
          (truly-the ,elt-type (data-vector-set
-                               (the simple-vector ,n-vector)
+                               (the* (simple-vector :modifying (setf svref)) ,n-vector)
                                (check-bound ,n-vector (length ,n-vector) ,index)
                                (the ,elt-type ,value)))))))
 
@@ -1707,7 +1759,9 @@
 ;;; Pick off some constant cases.
 (defoptimizer (array-header-p derive-type) ((array))
   (let ((type (lvar-type array)))
-    (cond ((not (array-type-p type))
+    (cond ((not (types-equal-or-intersect type (specifier-type 'simple-array)))
+           (specifier-type '(eql t)))
+          ((not (array-type-p type))
            ;; FIXME: use analogue of ARRAY-TYPE-DIMENSIONS-OR-GIVE-UP
            nil)
           (t
