@@ -111,7 +111,7 @@
   (always-translatable flushable)
   :result-arg 0)
 
-(defknown vector-fill* (t t t t) vector
+(defknown (vector-fill* vector-fill/t) (t t t t) vector
   ()
   :result-arg 0)
 
@@ -122,6 +122,9 @@
 (defknown vector-sap ((simple-unboxed-array (*))) system-area-pointer
   (flushable))
 
+#!+gencgc
+(defknown generation-of (t) (or (signed-byte 8) null) (flushable movable))
+
 ;;; WIDETAG-OF needs extra code to handle LIST and FUNCTION lowtags.
 ;;; When dealing with known other-pointers (dispatching on array
 ;;; element type for example), %OTHER-POINTER-WIDETAG is faster.
@@ -131,9 +134,14 @@
 
 ;;; Return the data from the header of object, which for GET-HEADER-DATA
 ;;; must be an other-pointer, and for GET-CLOSURE-LENGTH a fun-pointer.
-(defknown (get-header-data get-closure-length) (t)
+(defknown (get-header-data) (t)
     (unsigned-byte #.(- sb!vm:n-word-bits sb!vm:n-widetag-bits))
   (flushable))
+;;; Closures have at least a trampoline word (length can't be 0),
+;;; and only 15 bits in which to express the payload size.
+(defknown (get-closure-length) (t) (integer 1 #.sb!vm:short-header-max-words)
+  (flushable))
+
 ;;; This unconventional setter returns its first arg, not the newval.
 (defknown set-header-data
     (t (unsigned-byte #.(- sb!vm:n-word-bits sb!vm:n-widetag-bits))) t)
