@@ -8,6 +8,14 @@
 ;;;; files for more information.
 
 (in-package "SB!VM")
+
+(defconstant sb!assem:assem-scheduler-p t)
+
+(defconstant +backend-fasl-file-implementation+ :mips)
+
+  ;; The o32 ABI specifies 4k-64k as page size. We have to pick the
+  ;; maximum since mprotect() works only with page granularity.
+(defconstant +backend-page-bytes+ 65536)
 
 ;;;; Machine Architecture parameters:
 (eval-when (:compile-toplevel :load-toplevel :execute)
@@ -75,10 +83,8 @@
   (defconstant static-space-start    #x06000000)
   (defconstant static-space-end      #x08000000)
 
-  (defconstant dynamic-0-space-start #x08000000)
-  (defconstant dynamic-0-space-end   #x0c000000)
-  (defconstant dynamic-1-space-start #x0c000000)
-  (defconstant dynamic-1-space-end   #x10000000))
+  (defparameter dynamic-0-space-start #x08000000)
+  (defparameter dynamic-0-space-end   #x0c000000))
 
 #!+linux
 (progn
@@ -92,10 +98,8 @@
   (defconstant static-space-end      #x0fff0000)
   ;; C runtime read/write segment starts at 0x10000000, heap and DSOs
   ;; start at 0x2a000000
-  (defconstant dynamic-0-space-start #x30000000)
-  (defconstant dynamic-0-space-end   #x4fff0000)
-  (defconstant dynamic-1-space-start #x50000000)
-  (defconstant dynamic-1-space-end   #x6fff0000)
+  (defparameter dynamic-0-space-start #x30000000)
+  (defparameter dynamic-0-space-end   #x4fff0000)
 
   (defconstant linkage-table-space-start #x70000000)
   (defconstant linkage-table-space-end   #x71000000)
@@ -116,13 +120,13 @@
 (defenum (:start 8)
   halt-trap
   pending-interrupt-trap
-  error-trap
   cerror-trap
   breakpoint-trap
   fun-end-breakpoint-trap
   after-breakpoint-trap
   single-step-around-trap
-  single-step-before-trap)
+  single-step-before-trap
+  error-trap)
 
 ;;;; Static symbols.
 
