@@ -24,8 +24,7 @@
 
 ;;;; conflict determination
 
-;;; Return true if the element at the specified offset, or in any of
-;;; the [size-1] subsequent offsets, in SB has a conflict with TN:
+;;; Return true if TN has a conflict in SC at the specified offset.
 ;;; -- If a component-live TN (:COMPONENT kind), then iterate over
 ;;;    all the blocks. If the element at OFFSET is used anywhere in
 ;;;    any of the component's blocks (always-live /= 0), then there
@@ -41,12 +40,14 @@
 ;;;    it is local to.
 ;;;
 ;;; If there is a conflict, returns the first such conflicting offset.
-(defun offset-conflicts-in-sb (tn sb offset &key (size 1))
-  (declare (type tn tn) (type finite-sb sb) (type index offset size))
-  (let ((confs (tn-global-conflicts tn))
-        (kind (tn-kind tn))
-        (sb-conflicts (finite-sb-conflicts sb))
-        (sb-always-live (finite-sb-always-live sb)))
+(declaim (ftype (sfunction (tn storage-class index) (or null index)) conflicts-in-sc))
+(defun conflicts-in-sc (tn sc offset)
+  (let* ((confs (tn-global-conflicts tn))
+         (kind (tn-kind tn))
+         (sb (sc-sb sc))
+         (sb-conflicts (finite-sb-conflicts sb))
+         (sb-always-live (finite-sb-always-live sb))
+         (size (sc-element-size sc)))
     (macrolet ((do-offsets ((var) &body body)
                  `(loop for ,var upfrom offset
                         repeat size
@@ -69,13 +70,13 @@
                  (do-offsets (offset-iter)
                      (let ((loc-live (svref sb-always-live offset-iter)))
                        (when (/= (sbit loc-live num) 0)
-                         (return-from offset-conflicts-in-sb offset-iter))))
+                         (return-from conflicts-in-sc offset-iter))))
                  (do-offsets (offset-iter)
                      (let ((loc-confs (svref sb-conflicts offset-iter)))
                        (when (/= (sbit (svref loc-confs num)
                                        (global-conflicts-number conf))
                                  0)
-                         (return-from offset-conflicts-in-sb offset-iter))))))))
+                         (return-from conflicts-in-sc offset-iter))))))))
         (t
          (do-offsets (offset-iter)
              (and (/= (sbit (svref (svref sb-conflicts offset-iter)
@@ -84,13 +85,6 @@
                       0)
                   offset-iter)))))))
 
-;;; Return true if TN has a conflict in SC at the specified offset.
-(declaim (ftype (sfunction (tn sc index) (or null index)) conflicts-in-sc))
-(defun conflicts-in-sc (tn sc offset)
-  (declare (type tn tn) (type sc sc) (type index offset))
-  (offset-conflicts-in-sb tn (sc-sb sc) offset
-                          :size (sc-element-size sc)))
-
 ;;; Add TN's conflicts into the conflicts for the location at OFFSET
 ;;; in SC. We iterate over each location in TN, adding to the
 ;;; conflicts for that location:
@@ -106,7 +100,7 @@
 ;;; -- If the TN is local, then we just do the block it is local to,
 ;;;    setting always-live and OR'ing in the local conflicts.
 (defun add-location-conflicts (tn sc offset optimize)
-  (declare (type tn tn) (type sc sc) (type index offset))
+  (declare (type tn tn) (type storage-class sc) (type index offset))
   (let ((confs (tn-global-conflicts tn))
         (sb (sc-sb sc))
         (kind (tn-kind tn)))
@@ -150,7 +144,7 @@
 ;; offset has been marked as ALWAYS-LIVE.
 (defun find-location-usage (sb offset)
   (declare (optimize speed))
-  (declare (type sb sb) (type index offset))
+  (declare (type storage-base sb) (type index offset))
   (let* ((always-live (svref (finite-sb-always-live sb) offset)))
     (declare (simple-bit-vector always-live))
     (count 1 always-live)))
@@ -224,7 +218,7 @@
 ;;; the SC element size, whichever is larger. If NEEDED-SIZE is
 ;;; larger, then use that size.
 (defun grow-sc (sc &optional (needed-size 0))
-  (declare (type sc sc) (type index needed-size))
+  (declare (type storage-class sc) (type index needed-size))
   (let* ((sb (sc-sb sc))
          (size (finite-sb-current-size sb))
          (align-mask (1- (sc-alignment sc)))
@@ -383,8 +377,7 @@
       (let* ((sb (sc-sb sc))
              (confs (finite-sb-live-tns sb)))
         (aver (eq (sb-kind sb) :finite))
-        (dolist (el (sc-locations sc))
-          (declare (type index el))
+        (do-sc-locations (el (sc-locations sc) nil (sc-element-size sc))
           (let ((conf (load-tn-conflicts-in-sc op sc el t)))
             (if conf
                 (used (describe-tn-use el conf op))
@@ -798,6 +791,18 @@
           (when (eq (vop-info-save-p (vop-info vop)) t)
             (let ((penalty (+ save-penalty (vop-depth-cost vop))))
               (do-live-tns (tn (vop-save-set vop) block)
+                #!-fp-and-pc-standard-save
+                (let ((save-tn (tn-save-tn tn)))
+                  (when (and save-tn (eq :specified-save (tn-kind save-tn)))
+                    ;; If we're expecting to spill a TN with a
+                    ;; specified save slot (the OCFP or LRA save TNs),
+                    ;; force it to the stack now.  If we don't, then
+                    ;; it might not end up on the stack, which may
+                    ;; lead to some useless loads in some code, and
+                    ;; will lead to race conditions in the debugger
+                    ;; involving backtraces from asynchronous
+                    ;; interrupts.
+                    (setf (tn-sc tn) (tn-sc save-tn))))
                 (decf (tn-cost tn) penalty))))))
 
       (do ((tn (ir2-component-normal-tns (component-info component))
@@ -813,6 +818,28 @@
             (incf cost (+ write-cost (vop-depth-cost (tn-ref-vop ref)))))
           (setf (tn-cost tn) cost))))))
 
+;;; If we're not assigning costs, and on a system where it matters, go
+;;; through and force TNs with specified save locations (OCFP and LRA
+;;; save locations) to the stack if they are going to be spilled.  See
+;;; the comment in ASSIGN-TN-COSTS for consequences of not doing so.
+#!-fp-and-pc-standard-save
+(defun maybe-force-specified-saves-to-stack (component)
+  (do-ir2-blocks (block component)
+    (do ((vop (ir2-block-start-vop block) (vop-next vop)))
+        ((null vop))
+      (when (eq (vop-info-save-p (vop-info vop)) t)
+        (do-live-tns (tn (vop-save-set vop) block)
+          (let ((save-tn (tn-save-tn tn)))
+            (when (and save-tn (eq :specified-save (tn-kind save-tn)))
+              ;; If we're expecting to spill a TN with a
+              ;; specified save slot (the OCFP or LRA save TNs),
+              ;; force it to the stack now.  If we don't, then
+              ;; it might not end up on the stack, which may
+              ;; lead to some useless loads in some code, and
+              ;; race conditions in the debugger involving
+              ;; backtraces from asynchronous interrupts.
+              (setf (tn-sc tn) (tn-sc save-tn)))))))))
+
 ;;; Iterate over the normal TNs, folding over the depth of the looops
 ;;; that the TN is used in and storing the result in TN-LOOP-DEPTH.
 ;;: reducer is the function used to join depth values together. #'max
@@ -902,43 +929,45 @@
       ((eq current vop)
        (do ((res (vop-results vop) (tn-ref-across res)))
            ((null res))
-         (let* ((tn (tn-ref-tn res))
-                (sc (tn-sc tn))
-                (sb (sc-sb sc)))
-           (when (eq (sb-kind sb) :finite)
+         (unless (eq (tn-kind (tn-ref-tn res)) :unused)
+           (let* ((tn (tn-ref-tn res))
+                  (sc (tn-sc tn))
+                  (sb (sc-sb sc)))
+             (when (eq (sb-kind sb) :finite)
+               (do ((offset (tn-offset tn) (1+ offset))
+                    (end (+ (tn-offset tn) (sc-element-size sc))))
+                   ((= offset end))
+                 (declare (type index offset end))
+                 (setf (svref (finite-sb-live-tns sb) offset) nil)))))))
+    (do ((ref (vop-refs current) (tn-ref-next-ref ref)))
+        ((null ref))
+      (unless (eq (tn-kind (tn-ref-tn ref)) :unused)
+       (let ((ltn (tn-ref-load-tn ref)))
+         (when ltn
+           (let* ((sc (tn-sc ltn))
+                  (sb (sc-sb sc)))
+             (when (eq (sb-kind sb) :finite)
+               (let ((tns (finite-sb-live-tns sb)))
+                 (do ((offset (tn-offset ltn) (1+ offset))
+                      (end (+ (tn-offset ltn) (sc-element-size sc))))
+                     ((= offset end))
+                   (declare (type index offset end))
+                   (aver (null (svref tns offset)))))))))
+
+       (let* ((tn (tn-ref-tn ref))
+              (sc (tn-sc tn))
+              (sb (sc-sb sc)))
+         (when (eq (sb-kind sb) :finite)
+           (let ((tns (finite-sb-live-tns sb)))
              (do ((offset (tn-offset tn) (1+ offset))
                   (end (+ (tn-offset tn) (sc-element-size sc))))
                  ((= offset end))
                (declare (type index offset end))
-               (setf (svref (finite-sb-live-tns sb) offset) nil))))))
-    (do ((ref (vop-refs current) (tn-ref-next-ref ref)))
-        ((null ref))
-      (let ((ltn (tn-ref-load-tn ref)))
-        (when ltn
-          (let* ((sc (tn-sc ltn))
-                 (sb (sc-sb sc)))
-            (when (eq (sb-kind sb) :finite)
-              (let ((tns (finite-sb-live-tns sb)))
-                (do ((offset (tn-offset ltn) (1+ offset))
-                     (end (+ (tn-offset ltn) (sc-element-size sc))))
-                    ((= offset end))
-                  (declare (type index offset end))
-                  (aver (null (svref tns offset)))))))))
-
-      (let* ((tn (tn-ref-tn ref))
-             (sc (tn-sc tn))
-             (sb (sc-sb sc)))
-        (when (eq (sb-kind sb) :finite)
-          (let ((tns (finite-sb-live-tns sb)))
-            (do ((offset (tn-offset tn) (1+ offset))
-                 (end (+ (tn-offset tn) (sc-element-size sc))))
-                ((= offset end))
-              (declare (type index offset end))
-              (if (tn-ref-write-p ref)
-                  (setf (svref tns offset) nil)
-                  (let ((old (svref tns offset)))
-                    (aver (or (null old) (eq old tn)))
-                    (setf (svref tns offset) tn)))))))))
+               (if (tn-ref-write-p ref)
+                   (setf (svref tns offset) nil)
+                   (let ((old (svref tns offset)))
+                     (aver (or (null old) (eq old tn)))
+                     (setf (svref tns offset) tn))))))))))
 
   (setq *live-vop* vop)
   (values))
@@ -969,7 +998,7 @@
 ;;;
 ;;; We return a conflicting TN if there is a conflict.
 (defun load-tn-offset-conflicts-in-sb (op sb offset)
-  (declare (type tn-ref op) (type finite-sb sb) (type index offset))
+  (declare (type tn-ref op) (type sb!c::finite-sb-template sb) (type index offset))
   (aver (eq (sb-kind sb) :finite))
   (let ((vop (tn-ref-vop op)))
     (labels ((tn-overlaps (tn)
@@ -984,8 +1013,9 @@
              (same (ref)
                (let ((tn (tn-ref-tn ref))
                      (ltn (tn-ref-load-tn ref)))
-                 (or (tn-overlaps tn)
-                     (and ltn (tn-overlaps ltn)))))
+                 (and (not (eq (tn-kind tn) :unused))
+                      (or (tn-overlaps tn)
+                          (and ltn (tn-overlaps ltn))))))
              (is-op (ops)
                (do ((ops ops (tn-ref-across ops)))
                    ((null ops) nil)
@@ -1044,7 +1074,8 @@
       (let* ((tn (tn-ref-tn target))
              (loc (tn-offset tn)))
         (if (and (eq (tn-sc tn) sc)
-                 (member (the index loc) (sc-locations sc))
+                 (sc-locations-member (the sb!vm:finite-sc-offset loc)
+                                      (sc-locations sc))
                  (not (load-tn-conflicts-in-sc op sc loc nil)))
             loc
             nil)))))
@@ -1053,7 +1084,7 @@
 ;;; iterate over the SC's locations. If we can't find a legal
 ;;; location, return NIL.
 (defun select-load-tn-location (op sc)
-  (declare (type tn-ref op) (type sc sc))
+  (declare (type tn-ref op) (type storage-class sc))
 
   ;; Check any target location first.
   (let ((target (tn-ref-target op)))
@@ -1061,11 +1092,12 @@
       (let* ((tn (tn-ref-tn target))
              (loc (tn-offset tn)))
         (when (and (eq (sc-sb sc) (sc-sb (tn-sc tn)))
-                   (member (the index loc) (sc-locations sc))
+                   (sc-locations-member (the sb!vm:finite-sc-offset loc)
+                                        (sc-locations sc))
                    (not (load-tn-conflicts-in-sc op sc loc nil)))
               (return-from select-load-tn-location loc)))))
 
-  (dolist (loc (sc-locations sc) nil)
+  (do-sc-locations (loc (sc-locations sc) nil (sc-element-size sc))
     (unless (load-tn-conflicts-in-sc op sc loc nil)
       (return loc))))
 
@@ -1107,7 +1139,7 @@
 ;;; argument or result TN. The only way we can fail is if all
 ;;; locations in SC are used by load-TNs or temporaries in VOP.
 (defun unpack-for-load-tn (sc op)
-  (declare (type sc sc) (type tn-ref op))
+  (declare (type storage-class sc) (type tn-ref op))
   (let ((sb (sc-sb sc))
         (normal-tns (ir2-component-normal-tns
                      (component-info *component-being-compiled*)))
@@ -1119,8 +1151,7 @@
                (event unpack-tn node)
                (unpack-tn victim))
              (throw 'unpacked-tn nil)))
-      (dolist (loc (sc-locations sc))
-        (declare (type index loc))
+      (do-sc-locations (loc (sc-locations sc) nil (sc-element-size sc))
         (block SKIP
           (collect ((victims nil adjoin))
             (do ((i loc (1+ i))
@@ -1204,32 +1235,36 @@
   (do ((scs scs (cdr scs))
        (op ops (tn-ref-across op)))
       ((null scs))
-      (let ((target (tn-ref-target op)))
-        (when target
-           (let* ((load-tn (tn-ref-load-tn op))
-                  (load-scs (svref (car scs)
-                                   (sc-number
-                                    (tn-sc (or load-tn (tn-ref-tn op)))))))
-             (if load-tn
-                 (aver (eq load-scs t))
-               (unless (eq load-scs t)
-                       (setf (tn-ref-load-tn op)
-                             (pack-load-tn (car scs) op))))))))
+    (let ((target (tn-ref-target op))
+          (tn (tn-ref-tn op)))
+      (when (and target
+                 (not (eq (tn-kind tn) :unused)))
+        (let* ((load-tn (tn-ref-load-tn op))
+               (load-scs (svref (car scs)
+                                (sc-number
+                                 (tn-sc (or load-tn (tn-ref-tn op)))))))
+          (if load-tn
+              (aver (eq load-scs t))
+              (unless (eq load-scs t)
+                (setf (tn-ref-load-tn op)
+                      (pack-load-tn (car scs) op))))))))
 
   (do ((scs scs (cdr scs))
        (op ops (tn-ref-across op)))
       ((null scs))
-      (let ((target (tn-ref-target op)))
-        (unless target
-           (let* ((load-tn (tn-ref-load-tn op))
-                  (load-scs (svref (car scs)
-                                   (sc-number
-                                    (tn-sc (or load-tn (tn-ref-tn op)))))))
-             (if load-tn
-                 (aver (eq load-scs t))
-               (unless (eq load-scs t)
-                       (setf (tn-ref-load-tn op)
-                             (pack-load-tn (car scs) op))))))))
+    (let ((target (tn-ref-target op))
+          (tn (tn-ref-tn op)))
+      (unless (or target
+                  (eq (tn-kind tn) :unused))
+        (let* ((load-tn (tn-ref-load-tn op))
+               (load-scs (svref (car scs)
+                                (sc-number
+                                 (tn-sc (or load-tn tn))))))
+          (if load-tn
+              (aver (eq load-scs t))
+              (unless (eq load-scs t)
+                (setf (tn-ref-load-tn op)
+                      (pack-load-tn (car scs) op))))))))
 
   (values))
 
@@ -1261,14 +1296,16 @@
   ;; the write and vice versa.
   (aver (eq (tn-ref-write-p read)
             (not (tn-ref-write-p write))))
-  (setf (tn-ref-target read) write)
-  (setf (tn-ref-target write) read))
+  (unless (or (eq (tn-kind (tn-ref-tn write)) :unused)
+              (eq (tn-kind (tn-ref-tn read)) :unused))
+    (setf (tn-ref-target read) write)
+    (setf (tn-ref-target write) read)))
 
 ;;; If TN can be packed into SC so as to honor a preference to TARGET,
 ;;; then return the offset to pack at, otherwise return NIL. TARGET
 ;;; must be already packed.
 (defun check-ok-target (target tn sc)
-  (declare (type tn target tn) (type sc sc) (inline member))
+  (declare (type tn target tn) (type storage-class sc) (inline member))
   (let* ((loc (tn-offset target))
          (target-sc (tn-sc target))
          (target-sb (sc-sb target-sc)))
@@ -1279,7 +1316,7 @@
     ;; -- TN doesn't conflict with target's location.
     (if (and (eq target-sb (sc-sb sc))
              (or (eq (sb-kind target-sb) :unbounded)
-                 (member loc (sc-locations sc)))
+                 (sc-locations-member loc (sc-locations sc)))
              (= (sc-element-size target-sc) (sc-element-size sc))
              (not (conflicts-in-sc tn sc loc))
              (zerop (mod loc (sc-alignment sc))))
@@ -1334,9 +1371,10 @@
        (%call-with-target-tns ,source-tn #',callback ,@keys))))
 
 (defun find-ok-target-offset (tn sc)
-  (declare (type tn tn) (type sc sc))
+  (declare (type tn tn) (type storage-class sc))
   (do-target-tns (target tn)
     (awhen (and (tn-offset target)
+                (neq (tn-kind target) :arg-pass)
                 (check-ok-target target tn sc))
       (return-from find-ok-target-offset it))))
 
@@ -1355,38 +1393,50 @@
 ;;; on register-starved architectures (x86) this seems to be a bad
 ;;; strategy. -- JES 2004-09-11
 (defun select-location (tn sc &key use-reserved-locs optimize)
-  (declare (type tn tn) (type sc sc) (inline member))
+  (declare (type tn tn) (type storage-class sc))
   (let* ((sb (sc-sb sc))
          (element-size (sc-element-size sc))
          (alignment (sc-alignment sc))
          (align-mask (1- alignment))
          (size (finite-sb-current-size sb)))
-    (flet ((attempt-location (start-offset)
+    (flet ((available-locations ()
+             (if use-reserved-locs
+                 (sc-locations sc)
+                 (logand (sc-locations sc)
+                         (lognot (sc-reserve-locations sc)))))
+           (location-cost (location-offset)
+             (let ((always-live-count (finite-sb-always-live-count sb)))
+               (loop for offset from location-offset
+                  repeat element-size
+                  maximize (svref always-live-count offset))))
+           (attempt-location (start-offset)
              (let ((conflict (conflicts-in-sc tn sc start-offset)))
                (if conflict
                    (logandc2 (+ conflict align-mask 1)
                              align-mask)
                    (return-from select-location start-offset)))))
-      (if (eq (sb-kind sb) :unbounded)
-          (loop with offset = 0
-                until (> (+ offset element-size) size) do
-                (setf offset (attempt-location offset)))
-          (let ((locations (sc-locations sc)))
-            (when optimize
-              (setf locations
-                    (schwartzian-stable-sort-list
-                     locations '>
-                     :key (lambda (location-offset)
-                            (loop for offset from location-offset
-                                  repeat element-size
-                                  maximize (svref
-                                            (finite-sb-always-live-count sb)
-                                            offset))))))
-            (dolist (offset locations)
-              (when (or use-reserved-locs
-                        (not (member offset
-                                     (sc-reserve-locations sc))))
-                (attempt-location offset))))))))
+      (cond
+        ((eq (sb-kind sb) :unbounded)
+         (loop with offset = 0
+            until (> (+ offset element-size) size) do
+              (setf offset (attempt-location offset))))
+        (optimize ; finite SB, optimizing
+         ;; Sort locations available in the SC by LOCATION-COST
+         (let* ((locations (available-locations))
+                (locations+costs (make-array (sc-locations-count locations))))
+           (declare (dynamic-extent locations+costs))
+           (let ((i 0))
+             (do-sc-locations (location locations nil (sc-element-size sc))
+               (setf (aref locations+costs i)
+                     (cons location (location-cost location)))
+               (incf i)))
+           (setq locations+costs (sort locations+costs #'> :key #'cdr))
+           (dovector (location-and-cost locations+costs)
+             (attempt-location (car location-and-cost)))))
+        (t ; finite SB, not optimizing
+         (do-sc-locations (location (available-locations) nil
+                                    (sc-element-size sc))
+           (attempt-location location)))))))
 
 ;;; If a save TN, return the saved TN, otherwise return TN. This is
 ;;; useful for getting the conflicts of a TN that might be a save TN.
@@ -1483,7 +1533,7 @@
     ;; For non-x86 ports the presence of a save-tn associated with a
     ;; tn is used to identify the old-fp and return-pc tns. It depends
     ;; on the old-fp and return-pc being passed in registers.
-    #!-(or x86 x86-64 arm arm64)
+    #!-fp-and-pc-standard-save
     (when (and (not (eq (tn-kind tn) :specified-save))
                (conflicts-in-sc original sc offset))
       (error "~S is wired to a location that it conflicts with." tn))
@@ -1510,13 +1560,13 @@
     ;; the stack so the above hack for the other ports does not always
     ;; work. Here the old-fp and return-pc tns are identified by being
     ;; on the stack in their standard save locations.
-    #!+(or x86 x86-64 arm arm64)
-    (when (and (not (eq (tn-kind tn) :specified-save))
-               (not (and (string= (sb-name sb)
-                                  #!-(or arm arm64) "STACK"
-                                  #!+(or arm arm64) "CONTROL-STACK")
-                         (or (= offset 0)
-                             (= offset 1))))
+    #!+fp-and-pc-standard-save
+    (when (and (not (and
+                     (= (sc-number sc) #.(sc+offset-scn old-fp-passing-offset))
+                     (= offset #.(sc+offset-offset old-fp-passing-offset))))
+               (not (and
+                     (= (sc-number sc) #.(sc+offset-scn return-pc-passing-offset))
+                     (= offset #.(sc+offset-offset return-pc-passing-offset))))
                (conflicts-in-sc original sc offset))
       (error "~S is wired to location ~D in SC ~A of kind ~S that it conflicts with."
              tn offset sc (tn-kind tn)))
@@ -1624,8 +1674,10 @@
 
          ;; Assign costs to normal TNs so we know which ones should always
          ;; be packed on the stack, and which are important not to spill.
-         (when *pack-assign-costs*
-           (assign-tn-costs component))
+         (if *pack-assign-costs*
+             (assign-tn-costs component)
+             #!-fp-and-pc-standard-save
+             (maybe-force-specified-saves-to-stack component))
 
          ;; Actually allocate registers for most TNs. After this, only
          ;; :normal tns may be left unallocated (or TNs :restricted to
@@ -1705,7 +1757,8 @@
   ;; Pack wired TNs first.
   (do ((tn (ir2-component-wired-tns 2comp) (tn-next tn)))
       ((null tn))
-    (pack-wired-tn tn optimize))
+    (unless (eq (tn-kind tn) :arg-pass)
+      (pack-wired-tn tn optimize)))
 
   ;; Then, pack restricted TNs, ones that are live over the whole
   ;; component first (they cause no fragmentation).  Sort by TN cost
