@@ -17,7 +17,7 @@
 ;;; from the C alloc() function by way of the alloc-tramp
 ;;; assembly routine.
 
-(defun allocation-dynamic-extent (alloc-tn size lowtag)
+(defun stack-allocation (alloc-tn size lowtag)
   (aver (not (location= alloc-tn rsp-tn)))
   (inst sub rsp-tn size)
   ;; see comment in x86/macros.lisp implementation of this
@@ -31,7 +31,7 @@
   (inst lea alloc-tn (make-ea :byte :base rsp-tn :disp lowtag))
   (values))
 
-(defun allocation-tramp (node result-tn size lowtag)
+(defun %alloc-tramp (node result-tn size lowtag)
   (cond ((typep size '(and integer (not (signed-byte 32))))
          ;; MOV accepts large immediate operands, PUSH does not
          (inst mov result-tn size)
@@ -51,50 +51,10 @@
     (inst or (reg-in-size result-tn :byte) lowtag))
   (values))
 
-;;; Emit code to allocate an object with a size in bytes given by
-;;; SIZE into ALLOC-TN. The size may be an integer of a TN.
-;;; NODE may be used to make policy-based decisions.
-;;; This function should only be used inside a pseudo-atomic section,
-;;; which to the degree needed should also cover subsequent initialization.
-(defun allocation (alloc-tn size node &optional dynamic-extent lowtag)
-  (when dynamic-extent
-    (allocation-dynamic-extent alloc-tn size lowtag)
-    (return-from allocation (values)))
-  (aver (and (not (location= alloc-tn temp-reg-tn))
-             (or (integerp size) (not (location= size temp-reg-tn)))))
-
-  #!+(and (not sb-thread) sb-dynamic-core)
-  ;; We'd need a spare reg in which to load boxed_region from the linkage table.
-  ;; Could push/pop any random register on the stack and own it temporarily,
-  ;; but seeing as nobody cared about this, just punt.
-  (allocation-tramp node alloc-tn size lowtag)
-
-  #!-(and (not sb-thread) sb-dynamic-core)
-  ;; Otherwise do the normal inline allocation thing
-  (let ((NOT-INLINE (gen-label))
-        (DONE (gen-label))
-        (SKIP-INSTRUMENTATION (gen-label))
-        ;; Yuck.
-        (in-elsewhere (eq *elsewhere* sb!assem::**current-segment**))
-        ;; thread->alloc_region.free_pointer
-        (free-pointer
-         #!+sb-thread
-         (thread-tls-ea (* n-word-bytes thread-alloc-region-slot))
-         #!-sb-thread
-         (make-ea :qword :disp (make-fixup "gc_alloc_region" :foreign)))
-        ;; thread->alloc_region.end_addr
-        (end-addr
-         #!+sb-thread
-         (thread-tls-ea (* n-word-bytes (1+ thread-alloc-region-slot)))
-         #!-sb-thread
-         (make-ea :qword :disp (make-fixup "gc_alloc_region" :foreign 8))))
-
-    ;; Insert allocation profiler instrumentation
-    ;; FIXME: for now, change this to '>=' to perform self-build
-    ;;        where the resulting executable has instrumentation
-    ;;        because I can't get policies to work.
-    ;; FIXME: and does this work for assembly routines?
-    (when (policy node (> sb!c::instrument-consing 1))
+;;; Insert allocation profiler instrumentation
+(defun instrument-alloc (size node)
+  (when (policy node (> sb!c::instrument-consing 1))
+    (let ((skip-instrumentation (gen-label)))
       (inst mov temp-reg-tn
             (make-ea :qword :base thread-base-tn
                      :disp (* n-word-bytes thread-profile-data-slot)))
@@ -113,7 +73,7 @@
         (cond ((or (not node) ; assembly routine
                    (sb!c::code-immobile-p node))
                (inst call (make-fixup helper :assembly-routine)) ; 5 bytes
-               (emit-long-nop sb!assem::**current-segment** 3)) ; align
+               (emit-alignment 3 :long-nop))
               (t
                (inst call ; 7 bytes
                      (make-ea :qword :disp
@@ -123,14 +83,51 @@
           ;; This TEST instruction is never executed- it informs the profiler
           ;; which register holds SIZE.
           (inst test size size) ; 3 bytes
-          (emit-long-nop sb!assem::**current-segment** 5))) ; align
-      (emit-label skip-instrumentation))
+          (emit-alignment 3 :long-nop)))
+      (emit-label skip-instrumentation))))
+
+;;; Emit code to allocate an object with a size in bytes given by
+;;; SIZE into ALLOC-TN. The size may be an integer of a TN.
+;;; NODE may be used to make policy-based decisions.
+;;; This function should only be used inside a pseudo-atomic section,
+;;; which to the degree needed should also cover subsequent initialization.
+(defun allocation (alloc-tn size node &optional dynamic-extent lowtag)
+  (when dynamic-extent
+    (stack-allocation alloc-tn size lowtag)
+    (return-from allocation (values)))
+  (aver (and (not (location= alloc-tn temp-reg-tn))
+             (or (integerp size) (not (location= size temp-reg-tn)))))
+
+  #!+(and (not sb-thread) sb-dynamic-core)
+  ;; We'd need a spare reg in which to load boxed_region from the linkage table.
+  ;; Could push/pop any random register on the stack and own it temporarily,
+  ;; but seeing as nobody cared about this, just punt.
+  (%alloc-tramp node alloc-tn size lowtag)
+
+  #!-(and (not sb-thread) sb-dynamic-core)
+  ;; Otherwise do the normal inline allocation thing
+  (let ((NOT-INLINE (gen-label))
+        (DONE (gen-label))
+        ;; Yuck.
+        (in-elsewhere (sb!assem::assembling-to-elsewhere-p))
+        ;; thread->alloc_region.free_pointer
+        (free-pointer
+         #!+sb-thread
+         (thread-tls-ea (* n-word-bytes thread-alloc-region-slot))
+         #!-sb-thread
+         (make-ea :qword :disp (make-fixup "gc_alloc_region" :foreign)))
+        ;; thread->alloc_region.end_addr
+        (end-addr
+         #!+sb-thread
+         (thread-tls-ea (* n-word-bytes (1+ thread-alloc-region-slot)))
+         #!-sb-thread
+         (make-ea :qword :disp (make-fixup "gc_alloc_region" :foreign 8))))
 
     (cond ((or in-elsewhere
                ;; large objects will never be made in a per-thread region
                (and (integerp size)
                     (>= size large-object-size)))
-           (allocation-tramp node alloc-tn size lowtag))
+           (%alloc-tramp node alloc-tn size lowtag))
           (t
            (inst mov temp-reg-tn free-pointer)
            (cond ((integerp size)
@@ -146,30 +143,32 @@
            (if lowtag
                (inst lea alloc-tn (make-ea :byte :base temp-reg-tn :disp lowtag))
                (inst mov alloc-tn temp-reg-tn))
-           (assemble (*elsewhere*)
+           (assemble (:elsewhere)
              (emit-label NOT-INLINE)
              (cond ((and (tn-p size) (location= size alloc-tn)) ; recover SIZE
                     (inst sub alloc-tn free-pointer)
-                    (allocation-tramp node temp-reg-tn alloc-tn nil))
+                    (%alloc-tramp node temp-reg-tn alloc-tn nil))
                    (t ; SIZE is intact
-                    (allocation-tramp node temp-reg-tn size nil)))
+                    (%alloc-tramp node temp-reg-tn size nil)))
              (inst jmp DONE))))
     (values)))
 
 ;;; Allocate an other-pointer object of fixed SIZE with a single word
 ;;; header having the specified WIDETAG value. The result is placed in
 ;;; RESULT-TN.
-(defun fixed-alloc (result-tn widetag size node &optional stack-allocate-p)
+(defun fixed-alloc (result-tn widetag size node &optional stack-allocate-p
+                    &aux (bytes (pad-data-block size)))
+  (unless stack-allocate-p
+    (instrument-alloc bytes node))
   (maybe-pseudo-atomic stack-allocate-p
-      (allocation result-tn (pad-data-block size) node stack-allocate-p
-                  other-pointer-lowtag)
+      (allocation result-tn bytes node stack-allocate-p other-pointer-lowtag)
       (storew* (logior (ash (1- size) n-widetag-bits) widetag)
                result-tn 0 other-pointer-lowtag
                (not stack-allocate-p))))
 
 ;;;; CONS, LIST and LIST*
 (define-vop (list-or-list*)
-  (:args (things :more t))
+  (:args (things :more t :scs (descriptor-reg constant immediate)))
   (:temporary (:sc unsigned-reg) ptr temp)
   (:temporary (:sc unsigned-reg :to (:result 0) :target result) res)
   (:info num)
@@ -187,17 +186,22 @@
            (macrolet
                ((store-car (tn list &optional (slot cons-car-slot))
                   `(let ((reg
+                          ;; FIXME: single-float gets placed in the boxed header
+                          ;; rather than just doing an immediate store.
                           (sc-case ,tn
-                            ((any-reg descriptor-reg) ,tn)
-                            ((control-stack)
+                            ((control-stack constant)
                              (move temp ,tn)
-                             temp))))
+                             temp)
+                            (t
+                             (encode-value-if-immediate ,tn)))))
                      (storew reg ,list ,slot list-pointer-lowtag))))
-             (let ((cons-cells (if star (1- num) num))
-                   (stack-allocate-p (node-stack-allocate-p node)))
+             (let* ((cons-cells (if star (1- num) num))
+                    (stack-allocate-p (node-stack-allocate-p node))
+                    (size (* (pad-data-block cons-size) cons-cells)))
+               (unless stack-allocate-p
+                 (instrument-alloc size node))
                (maybe-pseudo-atomic stack-allocate-p
-                (allocation res (* (pad-data-block cons-size) cons-cells) node
-                            stack-allocate-p list-pointer-lowtag)
+                (allocation res size node stack-allocate-p list-pointer-lowtag)
                 (move ptr res)
                 (dotimes (i (1- cons-cells))
                   (store-car (tn-ref-tn things) ptr)
@@ -284,6 +288,7 @@
       ;; The LET generates instructions that needn't be pseudoatomic
       ;; so don't move it inside.
       (let ((size (calc-size-in-bytes words result)))
+        (instrument-alloc size node)
         (pseudo-atomic
          (allocation result size node nil other-pointer-lowtag)
          (put-header result type length t)))))
@@ -304,7 +309,7 @@
     (:generator 100
       (let ((size (calc-size-in-bytes words result))
             (rax-zeroed))
-        (allocation result size node t other-pointer-lowtag)
+        (stack-allocation result size other-pointer-lowtag)
         (put-header result type length nil)
         ;; FIXME: It would be good to check for stack overflow here.
         ;; It would also be good to skip zero-fill of specialized vectors
@@ -339,35 +344,38 @@
               (inst mov words result) ; restore 'words'
               (inst lea result ; recompute the tagged pointer
                     (make-ea :byte :base rsp-tn :disp other-pointer-lowtag)))))
-        (let ((data-addr
-                (make-ea :qword :base result
-                                :disp (- (* vector-data-offset n-word-bytes)
-                                         other-pointer-lowtag))))
-          (block zero-fill
-            (cond ((sc-is words immediate)
-                   (let ((n (tn-value words)))
-                     (cond ((> n 8)
-                            (inst mov rcx (tn-value words)))
-                           ((= n 1)
-                            (inst mov data-addr 0)
-                            (return-from zero-fill))
-                           (t
-                            (multiple-value-bind (double single) (truncate n 2)
-                              (inst xorpd zero zero)
-                              (dotimes (i double)
-                                (inst movapd data-addr zero)
-                                (setf data-addr (copy-structure data-addr))
-                                (incf (ea-disp data-addr) (* n-word-bytes 2)))
-                              (unless (zerop single)
-                                (inst movaps data-addr zero))
-                              (return-from zero-fill))))))
-                  (t
-                   (move rcx words)
-                   (inst shr rcx n-fixnum-tag-bits)))
-            (inst lea rdi data-addr)
-            (unless rax-zeroed (zeroize rax))
-            (inst rep)
-            (inst stos rax)))))))
+        (unless (sb!c::vector-initialized-p node)
+          (let ((data-addr
+                  (make-ea :qword :base result
+                                  :disp (- (* vector-data-offset n-word-bytes)
+                                           other-pointer-lowtag))))
+            (block zero-fill
+              (cond ((sc-is words immediate)
+                     (let ((n (tn-value words)))
+                       (cond ((> n 8)
+                              (inst mov rcx (tn-value words)))
+                             ((= n 1)
+                              (inst mov data-addr 0)
+                              (return-from zero-fill))
+                             (t
+                              (multiple-value-bind (double single) (truncate n 2)
+                                (inst xorpd zero zero)
+                                (dotimes (i double)
+                                  (inst movapd data-addr zero)
+                                  (setf data-addr
+                                        (make-ea :qword :base (ea-base data-addr)
+                                                        :disp (+ (ea-disp data-addr)
+                                                                 (* n-word-bytes 2)))))
+                                (unless (zerop single)
+                                  (inst movaps data-addr zero))
+                                (return-from zero-fill))))))
+                    (t
+                     (move rcx words)
+                     (inst shr rcx n-fixnum-tag-bits)))
+              (inst lea rdi data-addr)
+              (unless rax-zeroed (zeroize rax))
+              (inst rep)
+              (inst stos rax))))))))
 
 ;;; ALLOCATE-LIST
 (macrolet ((calc-size-in-bytes (length answer)
@@ -401,13 +409,11 @@
     (:results (result :scs (descriptor-reg) :from :load))
     (:arg-types positive-fixnum *)
     (:policy :fast-safe)
-    (:node-var node)
     (:temporary (:sc descriptor-reg) tail next limit)
-    (:node-var node)
     (:generator 20
       (let ((size (calc-size-in-bytes length next))
             (loop (gen-label)))
-        (allocation result size node t list-pointer-lowtag)
+        (stack-allocation result size list-pointer-lowtag)
         (compute-end)
         (inst mov next result)
         (emit-label LOOP)
@@ -437,6 +443,7 @@
             (loop (gen-label))
             (no-init
              (and (sc-is element immediate) (eql (tn-value element) 0))))
+        (instrument-alloc size node)
         (pseudo-atomic
          (allocation result size node nil list-pointer-lowtag)
          (compute-end)
@@ -475,11 +482,13 @@
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 10
-   (maybe-pseudo-atomic stack-allocate-p
-     (let* ((size (+ length closure-info-offset))
-            (header (logior (ash (1- size) n-widetag-bits) closure-widetag)))
-       (allocation result (pad-data-block size) node stack-allocate-p
-                   fun-pointer-lowtag)
+   (let* ((words (+ length closure-info-offset)) ; including header
+          (bytes (pad-data-block words))
+          (header (logior (ash (1- words) n-widetag-bits) closure-widetag)))
+     (unless stack-allocate-p
+       (instrument-alloc bytes node))
+     (maybe-pseudo-atomic stack-allocate-p
+       (allocation result bytes node stack-allocate-p fun-pointer-lowtag)
        (storew* #!-immobile-space header ; write the widetag and size
                 #!+immobile-space        ; ... plus the layout pointer
                 (progn (inst mov temp header)
@@ -493,13 +502,9 @@
      ;; This is due to scav_closure() assuming that it can always subtract
      ;; FUN_RAW_ADDR_OFFSET from closure->fun to obtain a Lisp object,
      ;; without any precheck for whether that word is currently 0.
-     (inst lea (reg-in-size temp :immobile-code-pc) (make-fixup nil :closure label))
-     (storew (reg-in-size temp
-                          (if stack-allocate-p
-                              ;; Need to do a full word store because the stack not zeroed
-                              :qword
-                              :immobile-code-pc))
-             result closure-fun-slot fun-pointer-lowtag))))
+     (inst lea temp (rip-relative-ea :qword label
+                                     (ash simple-fun-code-offset word-shift)))
+     (storew temp result closure-fun-slot fun-pointer-lowtag))))
 
 ;;; The compiler likes to be able to directly make value cells.
 (define-vop (make-value-cell)
@@ -535,9 +540,12 @@
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 50
+   (let ((bytes (pad-data-block words)))
     (progn name) ; possibly not used
+    (unless stack-allocate-p
+      (instrument-alloc bytes node))
     (maybe-pseudo-atomic stack-allocate-p
-     (allocation result (pad-data-block words) node stack-allocate-p lowtag)
+     (allocation result bytes node stack-allocate-p lowtag)
      (when type
        (let* ((widetag (if (typep type 'layout) instance-widetag type))
               (header (logior (ash (1- words) n-widetag-bits) widetag)))
@@ -552,7 +560,7 @@
          (unless (eq type widetag) ; TYPE is actually a LAYOUT
            (inst mov (make-ea :dword :base result :disp (+ 4 (- lowtag)))
                  ;; XXX: should layout fixups use a name, not a layout object?
-                 (make-fixup type :layout))))))))
+                 (make-fixup type :layout)))))))))
 
 ;;; Allocate a non-vector variable-length object.
 ;;; Exactly 4 allocators are rendered via this vop:
@@ -589,12 +597,17 @@
           (make-ea :qword :base header
                    :disp (+ (ash -2 n-widetag-bits) type)))
     (inst and bytes (lognot lowtag-mask)))
+    (instrument-alloc bytes node)
     (pseudo-atomic
      (allocation result bytes node nil lowtag)
      (storew header result 0 lowtag))))
 
 #!+immobile-space
-(progn
+(macrolet ((c-call (name)
+             `(let ((c-fun (make-fixup ,name :foreign)))
+                (inst call (cond ((sb!c::code-immobile-p node) c-fun)
+                                 (t (progn (inst mov temp-reg-tn c-fun)
+                                           temp-reg-tn)))))))
 (define-vop (alloc-immobile-fixedobj)
   (:info lowtag size word0 word1)
   (:temporary (:sc unsigned-reg :to :eval :offset rdi-offset) c-arg1)
@@ -602,6 +615,7 @@
   (:temporary (:sc unsigned-reg :from :eval :to (:result 0) :offset rax-offset)
               c-result)
   (:results (result :scs (descriptor-reg)))
+  (:node-var node)
   (:generator 50
    (inst mov c-arg1 size)
    (inst mov c-arg2 word0)
@@ -609,7 +623,7 @@
    ;; which has that effect
    (inst and rsp-tn -16)
    (pseudo-atomic
-     (inst call (make-fixup "alloc_fixedobj" :foreign))
+     (c-call "alloc_fixedobj")
      (inst lea result (make-ea :qword :base c-result :disp lowtag))
      ;; If code, the next word must be set within the P-A
      ;; otherwise the GC would compute the wrong object size.
@@ -622,13 +636,13 @@
   (:temporary (:sc unsigned-reg :from :eval :to (:result 0) :offset rax-offset)
               c-result)
   (:results (result :scs (descriptor-reg)))
+  (:node-var node)
   (:generator 50
    (move c-arg1 slots)
    ;; RSP needn't be restored because the allocators all return immediately
    ;; which has that effect
    (inst and rsp-tn -16)
-   (pseudo-atomic
-     (inst call (make-fixup "alloc_layout" :foreign)))
-     (move result c-result)))
-) ; end PROGN
+   (pseudo-atomic (c-call "alloc_layout"))
+   (move result c-result)))
+) ; end MACROLET
 
