@@ -180,11 +180,8 @@
   (:node-var node)
   (:note "float to pointer coercion")
   (:generator 13
-     (with-fixed-allocation (y
-                             double-float-widetag
-                             double-float-size
-                             node)
-       (inst movsd (ea-for-df-desc y) x))))
+     (fixed-alloc y double-float-widetag double-float-size node)
+     (inst movsd (ea-for-df-desc y) x)))
 (define-move-vop move-from-double :move
   (double-reg) (descriptor-reg))
 
@@ -240,11 +237,8 @@
   (:node-var node)
   (:note "complex float to pointer coercion")
   (:generator 13
-     (with-fixed-allocation (y
-                             complex-single-float-widetag
-                             complex-single-float-size
-                             node)
-       (inst movq (ea-for-csf-data-desc y) x))))
+     (fixed-alloc y complex-single-float-widetag complex-single-float-size node)
+     (inst movq (ea-for-csf-data-desc y) x)))
 (define-move-vop move-from-complex-single :move
   (complex-single-reg) (descriptor-reg))
 
@@ -254,11 +248,8 @@
   (:node-var node)
   (:note "complex float to pointer coercion")
   (:generator 13
-     (with-fixed-allocation (y
-                             complex-double-float-widetag
-                             complex-double-float-size
-                             node)
-       (inst movapd (ea-for-cdf-data-desc y) x))))
+     (fixed-alloc y complex-double-float-widetag complex-double-float-size node)
+     (inst movapd (ea-for-cdf-data-desc y) x)))
 (define-move-vop move-from-complex-double :move
   (complex-double-reg) (descriptor-reg))
 
@@ -400,7 +391,7 @@
                              ((eq (tn-kind arg) :constant)
                               (tn-value arg))
                              (t
-                              (make-sc-offset (sc-number (tn-sc arg))
+                              (make-sc+offset (sc-number (tn-sc arg))
                                               (or (tn-offset arg) 0))))))))
     (note-this-location vop :internal-error)))
 
@@ -1122,7 +1113,9 @@
                         (t))
                       (inst comiss x y))))))
   (define-</> < <single-float <double-float not :p :nc)
-  (define-</> > >single-float >double-float not :p :na))
+  (define-</> > >single-float >double-float not :p :na)
+  (define-</> <= <=single-float <=double-float not :p :a)
+  (define-</> >= >=single-float >=double-float not :p :b))
 
 
 ;;;; conversion
